#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from naruno.lib.log import get_logger
from naruno.transactions.pending.get_pending import GetPendingLen

logger = get_logger("BLOCKCHAIN")


def ChangeTransactionFee(
    block,
    custom_pending_transaction_len=None,
    custom_PENDING_TRANSACTIONS_PATH=None,
):
    """
    Increase transaction fee by 0.01 naruno coin for each block.default_optimum_transaction_number argument
    """
    logger.info("Calculating the transaction fee")
    logger.info(f"Start fee is: {block.transaction_fee}")
    pending_transactions = (GetPendingLen(
        custom_PENDING_TRANSACTIONS_PATH=custom_PENDING_TRANSACTIONS_PATH)
                            if custom_pending_transaction_len is None else
                            custom_pending_transaction_len)
    validating_list_len = 0
    for tx in block.validating_list:
        if tx.signature != "NARUNO":
            validating_list_len += 1
    total_len = validating_list_len + pending_transactions
    logger.debug(f"total_len: {total_len}")
    logger.debug(
        f"block.default_optimum_transaction_number {block.default_optimum_transaction_number}"
    )
    logger.debug(
        f"block.default_increase_of_fee {block.default_increase_of_fee}")
    if (total_len // block.default_optimum_transaction_number) != 0:
        increase = (total_len // block.default_optimum_transaction_number
                    ) * block.default_increase_of_fee
        block.transaction_fee += increase
        logger.info("Transaction fee will be increased")

    else:
        logger.info("Transaction fee is not changed")
        block.transaction_fee = block.default_transaction_fee
    logger.info(f"New transaction fee is : {block.transaction_fee} ")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from naruno.blockchain.block.block_main import Block
from naruno.blockchain.block.save_block import SaveBlock
from naruno.blockchain.candidate_block.candidate_block_main import \
    candidate_block
from naruno.consensus.rounds.round_1.checks.checks_main import \
    round_check
from naruno.consensus.rounds.round_1.process.process_main import \
    round_process
from naruno.lib.log import get_logger
from naruno.node.get_candidate_blocks import GetCandidateBlocks
from naruno.node.server.server import server
from naruno.node.unl import Unl
from naruno.transactions.pending_to_validating import PendingtoValidating

logger = get_logger("CONSENSUS_FIRST_ROUND")


def consensus_round_1(
    block: Block,
    custom_candidate_class: candidate_block = None,
    custom_unl_nodes: dict = None,
    custom_UNL_NODES_PATH: str = None,
    custom_server: server = None,
    custom_TEMP_ACCOUNTS_PATH: str = None,
    custom_TEMP_BLOCK_PATH: str = None,
    custom_TEMP_BLOCKSHASH_PATH: str = None,
    custom_TEMP_BLOCKSHASH_PART_PATH: str = None,
    custom_shares=None,
    custom_fee_address=None,
    clean=True,
) -> bool:
    """
    At this stage of the consensus process,
    The transactions of our and the unl nodes
    are evaluated and transactions accepted by
    owned by more than 50 percent.
    Inputs:
      * block: The block (class) we want consensus
      round 1 to be done
    """

    logger.info(
        f"BLOCK#{block.sequence_number}:{block.empty_block_number} First round is starting"
    )

    unl_nodes = (Unl.get_unl_nodes(custom_UNL_NODES_PATH=custom_UNL_NODES_PATH)
                 if custom_unl_nodes is None else custom_unl_nodes)
    logger.debug(f"unl_nodes: {unl_nodes}")
    candidate_class = (
        GetCandidateBlocks(custom_nodes_list=Unl.get_as_node_type(unl_nodes), block=block)
        if custom_candidate_class is None else custom_candidate_class)
    logger.debug(f"candidate_class: {candidate_class.__dict__}")

    if round_check(
        block, 
        candidate_class, 
        unl_nodes,
        custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH,
        custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
        custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH,
        custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH,        
        ):
        round_process(
            block,
            candidate_class,
            unl_nodes,
            custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH,
            custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
            custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH,
            custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH,
            custom_shares=custom_shares,
            custom_fee_address=custom_fee_address,
            clean=clean,
        )
        logger.info("Round 1 check is True")
        return True
    else:
       
        logger.warning("Round 1 check is False")
        return False



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os

from naruno.config import COMMANDERS_PATH
from naruno.lib.config_system import get_config
from naruno.transactions.transaction import Transaction


def GetCommander():
    the_pending_list = []
    os.chdir(get_config()["main_folder"])
    for entry in os.scandir(COMMANDERS_PATH):
        if entry.name != "README.md":
            with open(entry.path, "r") as my_transaction_file:
                the_pending_list.append(entry.name)

    # order the list by alphabetical order
    the_pending_list.sort()
    return the_pending_list



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import time

from naruno.blockchain.block.block_main import Block
from naruno.lib.log import get_logger

logger = get_logger("CONSENSUS_SECOND_ROUND")


def time_difference_check(block: Block) -> bool:

    true_time = block.round_2_starting_time + block.round_2_time
    current_time = int(time.time())
    logger.info("Round 2 time control started")
    logger.debug(f"current_time: {current_time}")
    logger.debug(f"true_time: {true_time}")
    if current_time >= true_time:
        logger.info("Time is true")
        return True
    else:
        logger.info("Time is not true")
        return False



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from urllib.request import urlopen

from naruno.blockchain.block.get_block import GetBlock
from naruno.lib.log import get_logger
from naruno.lib.settings_system import the_settings
from naruno.transactions.pending.get_pending import GetPendingLen

logger = get_logger("BLOCKCHAIN")


def GetMaxDataSize(block=None, ):
    if not the_settings()["baklava"]:
        block = block if block is not None else GetBlock()
        max_data_size = block.max_data_size
    else:
        max_data_size = int(
            urlopen("http://test_net.1.naruno.org:8000/blockmaxdatasize/get/").
            read().decode("utf-8"))

    return max_data_size



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import copy

from naruno.blockchain.block.block_main import Block
from naruno.transactions.check.check_transaction import \
    CheckTransaction
from naruno.transactions.check.datas.check_datas import Check_Datas
from naruno.transactions.pending.delete_pending import DeletePending
from naruno.transactions.pending.get_pending import GetPending
from naruno.transactions.pending.save_pending import SavePending


def Cleaner(block: Block, pending_list_txs: list,
    custom_current_time=None,
    custom_sequence_number=None,
    custom_balance=None,
):
    system_txs = []

    for transaction in block.validating_list:
        if transaction.signature == "NARUNO":
            block.validating_list.remove(transaction)
            system_txs.append(transaction)
            

    for transaction in pending_list_txs:
        the_sequance_number = None
        if custom_sequence_number == -1:
            the_sequance_number = transaction.sequence_number -1
        if not Check_Datas(
            block, 
            transaction, 
            custom_current_time=custom_current_time,
            custom_balance=custom_balance,
            custom_sequence_number=the_sequance_number,          
            disable_already_in=True):
            DeletePending(transaction)
            pending_list_txs.remove(transaction)

    for transaction in block.validating_list:
            the_sequance_number = None
            if custom_sequence_number == -1:
                the_sequance_number = transaction.sequence_number -1
            if not Check_Datas(
                block, 
                transaction,
                custom_current_time=custom_current_time,
                custom_balance=custom_balance,
                custom_sequence_number=the_sequance_number,            
                disable_already_in=True):
                block.validating_list.remove(transaction)

    def clean(list_of_transactions: list) -> list:
        list_of_transactions = list(dict.fromkeys(list_of_transactions))

        list_of_transactions = sorted(list_of_transactions,
                                      key=lambda x: x.signature)

        clean_list = []
        for transaction in list_of_transactions:
            ok = False
            just = True
            for transaction_ in list_of_transactions:
                if not transaction.__dict__ == transaction_.__dict__:
                    if transaction.fromUser == transaction_.fromUser and block.just_one_tx:
                        just = False
                        if transaction.sequence_number < transaction_.sequence_number:
                            ok = True
                        
                        elif (transaction.sequence_number ==
                              transaction_.sequence_number):
                            if (transaction.transaction_time <
                                    transaction_.transaction_time):
                                ok = True
                            elif (transaction.transaction_time ==
                                    transaction_.transaction_time):
                                if (transaction.signature <
                                        transaction_.signature):
                                    ok = True
                                else:
                                    ok = False
                                    break
                            else:
                                ok = False
                                break
                        else:
                            ok = False
                            break

            if ok or just:
                clean_list.append(transaction)

        return clean_list

    first_validating_list = copy.copy(block.validating_list)
    cleaned_validating_list = clean(block.validating_list)
    difference = list(
        set(first_validating_list) - set(cleaned_validating_list))
    for transaction in difference:
        SavePending(transaction)
    block.validating_list = cleaned_validating_list

    pending_list_txs = GetPending()
    first_pending_list_txs = copy.copy(pending_list_txs)
    cleaned_pending_list_txs = clean(pending_list_txs)
    difference = list(
        set(first_pending_list_txs) - set(cleaned_pending_list_txs))
    for transaction in difference:

        DeletePending(transaction)
    pending_list_txs = cleaned_pending_list_txs


    for transaction in system_txs:
        block.validating_list.append(transaction)


    return (cleaned_validating_list, cleaned_pending_list_txs)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from naruno.blockchain.candidate_block.candidate_block_main import \
    candidate_block
from naruno.lib.log import get_logger

logger = get_logger("CONSENSUS_SECOND_ROUND")


def candidate_blocks_hashes_check(candidate_class: candidate_block,
                                  unl_nodes: dict) -> bool:
    logger.info("Candidate block hash control started for round 2")
    logger.debug(
        f"len(candidate_class.candidate_block_hashes): {len(candidate_class.candidate_block_hashes)}"
    )
    logger.debug(f"len(unl_nodes): {len(unl_nodes)}")
    if len(candidate_class.candidate_block_hashes) > ((
        (len(unl_nodes) + 1) * 50) / 100):
        logger.info("Candidate block hash number is True")
        return True
    else:
        logger.info("Candidate block hash number is not True")
        return False



#!/usr/bin/python3
# -*- coding: utf-8 -*-
from naruno.wallet.ellipticcurve.curve import getCurveByOid
from naruno.wallet.ellipticcurve.curve import secp256k1
from naruno.wallet.ellipticcurve.math import Math
from naruno.wallet.ellipticcurve.point import Point
from naruno.wallet.ellipticcurve.utils.binary import base64FromByteString
from naruno.wallet.ellipticcurve.utils.binary import byteStringFromBase64
from naruno.wallet.ellipticcurve.utils.binary import byteStringFromHex
from naruno.wallet.ellipticcurve.utils.binary import hexFromByteString
from naruno.wallet.ellipticcurve.utils.binary import intFromHex
from naruno.wallet.ellipticcurve.utils.der import DerFieldType
from naruno.wallet.ellipticcurve.utils.der import encodeConstructed
from naruno.wallet.ellipticcurve.utils.der import encodePrimitive
from naruno.wallet.ellipticcurve.utils.der import hexFromInt
from naruno.wallet.ellipticcurve.utils.der import parse
from naruno.wallet.ellipticcurve.utils.pem import createPem
from naruno.wallet.ellipticcurve.utils.pem import getPemContent


class PublicKey:

    def __init__(self, point, curve):
        self.point = point
        self.curve = curve

    def toString(self, encoded=False):
        baseLength = 2 * self.curve.length()
        xHex = hexFromInt(self.point.x).zfill(baseLength)
        yHex = hexFromInt(self.point.y).zfill(baseLength)
        string = xHex + yHex
        if encoded:
            return f"0004{string}"

    def toDer(self):
        hexadecimal = encodeConstructed(
            encodeConstructed(
                encodePrimitive(DerFieldType.object, _ecdsaPublicKeyOid),
                encodePrimitive(DerFieldType.object, self.curve.oid),
            ),
            encodePrimitive(DerFieldType.bitString,
                            self.toString(encoded=True)),
        )
        return byteStringFromHex(hexadecimal)

    def toPem(self):
        der = self.toDer()
        return createPem(content=base64FromByteString(der),
                         template=_pemTemplate)

    @classmethod
    def fromPem(cls, string):
        publicKeyPem = getPemContent(pem=string, template=_pemTemplate)
        return cls.fromDer(byteStringFromBase64(publicKeyPem))

    @classmethod
    def fromDer(cls, string):
        hexadecimal = hexFromByteString(string)
        curveData, pointString = parse(hexadecimal)[0]
        publicKeyOid, curveOid = curveData

        curve = getCurveByOid(curveOid)
        return cls.fromString(string=pointString, curve=curve)

    @classmethod
    def fromString(cls, string, curve=secp256k1, validatePoint=True):
        baseLength = 2 * curve.length()
        if len(string) > 2 * baseLength and string[:4] == "0004":
            string = string[4:]

        xs = string[:baseLength]
        ys = string[baseLength:]

        p = Point(
            x=intFromHex(xs),
            y=intFromHex(ys),
        )
        publicKey = PublicKey(point=p, curve=curve)

        return publicKey


_ecdsaPublicKeyOid = (1, 2, 840, 10045, 2, 1)

_pemTemplate = """{content}"""



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from naruno.lib.log import get_logger
from naruno.lib.settings_system import the_settings

logger = get_logger("Safety")


def safety_check(
    interface=None,
    timeout=None,
    exit_on_error=True,
    custom_pywall=None,
    custom_debug_mode=None,
):
    logger.info("Checking safety")
    try:
        the_import_string = "from pywall import pywall"
        pywall_class = (exec(the_import_string)
                        if custom_pywall is None else custom_pywall)
        the_pywall = pywall() if custom_pywall is None else custom_pywall()
        the_pywall.iface = the_pywall.iface if interface is None else interface
        the_pywall.timeout = the_pywall.timeout if timeout is None else timeout

        control = the_pywall.control()

        debug_mode = (the_settings()["debug_mode"]
                      if custom_debug_mode is None else custom_debug_mode)

        if control is not None:
            if control:
                logger.info("NOT Safe")
                exit() if exit_on_error else None
                return False
            else:
                logger.info("Safe")
                return True
        elif debug_mode:
            logger.info(
                "Control check is none but passing because of debug mode")
            return None
        else:
            logger.info("NOT Safe (Control check is None)")
            exit() if exit_on_error else None
            return False
    except ImportError:
        logger.info("Passing safety check (no pywall)")
        return None



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from naruno.blockchain.block.hash.accounts_hash import AccountsHash
from naruno.blockchain.block.hash.blocks_hash import BlocksHash
from naruno.blockchain.block.hash.tx_hash import TransactionsHash
from naruno.consensus.rounds.round_1.process.transactions.checks.duplicated import \
    Remove_Duplicates
from naruno.lib.log import get_logger
from naruno.lib.mix.merkle_root import MerkleTree

logger = get_logger("BLOCKCHAIN")


def CalculateHash(block, part_of_blocks_hash, the_blocks_hash, the_accounts):
    """
    Calculates and returns the hash of the block.
    """
    logger.info(f"Calculating the hash of the block#{block.sequence_number}")

    block = Remove_Duplicates(block)
    block.validating_list = sorted(block.validating_list,
                                   key=lambda x: x.fromUser)

    # Transaction Hash
    validating_list = [tx.dump_json() for tx in block.validating_list]
    logger.debug(f"block.validating_list: {validating_list}")
    tx_hash = TransactionsHash(block)
    logger.debug(f"tx_hash: {tx_hash}")

    # Blocks Hash
    blocks_hash = BlocksHash(block, part_of_blocks_hash, the_blocks_hash)
    logger.debug(f"blocks_hash: {blocks_hash}")

    # Account
    accounts_hash = AccountsHash(block, the_accounts)
    logger.debug(f"accounts_hash: {accounts_hash}")

    # Other Elements
    main_list = [
        blocks_hash,
        accounts_hash,
        tx_hash,
        str(block.coin_amount),
        str(block.creator),
        str(block.genesis_time),
        str(block.block_time),
        str(block.previous_hash),
        str(block.sequence_number),
        str(block.hard_block_number),
        str(block.gap_block_number),
        str(block.transaction_fee),
        str(block.default_transaction_fee),
        str(block.default_optimum_transaction_number),
        str(block.default_increase_of_fee),
        str(block.transaction_delay_time),
        str(block.max_data_size),
        str(block.part_amount),
        str(block.max_tx_number),
        str(block.minumum_transfer_amount),
        str(block.round_1_time),
        str(block.round_2_time),
        # str(block.validated_time),
        str(block.just_one_tx),
        str(block.shares),
        str(block.fee_address),
    ]

    logger.debug(f"main_list: {main_list}")

    the_hash = MerkleTree(main_list).getRootHash()
    logger.debug(f"the_hash: {the_hash}")

    return the_hash



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import os

from kivymd.uix.button import MDFlatButton
from kivymd.uix.textfield import MDTextField
from kivymd_extensions.sweetalert import SweetAlert

import naruno.gui.the_naruno_gui_app


class popup:

    def __init__(
        self,
        title=None,
        text=None,
        image=None,
        height_image=None,
        thirdly_title=None,
        target=None,
        inputs=None,
        type="custom",
    ):
        """
        :param title: Title of the popup
        :param target: Function to be called when the OK button is pressed
        :param content: Content of the popup
        """
        self.title = title
        self.text = text
        self.image = image
        self.height_image = height_image

        self.thirdly_title = thirdly_title
        self.target = target
        self.inputs = inputs
        self.input_results = {}
        self.type = type
        self.dialog = None

        self.create()
        if self.type == "custom":
            self.show()

    def show(self):
        self.dialog.open()

    def dismiss(self, widget=None):
        self.dialog.dismiss()

    def clean(self):

        for obj in self.dialog.content_cls.children:
            if isinstance(obj, MDTextField):
                obj.text = ""

    def director(self, widget):
        print("director")

        for obj in self.dialog.content_cls.children:
            if isinstance(obj, MDTextField):
                self.input_results[obj.hint_text] = obj.text
        print("Target")
        self.target()
        print("Clean")
        self.clean()
        print("Dismiss")
        self.dismiss()
        print("Done")

    def director_without_input(self, widget):
        print("director")
        self.target()
        self.dismiss()

    def create(self):
        if self.dialog is None:
            if self.type == "custom":
                self.dialog = SweetAlert(
                    title=self.title,
                    type=self.type,
                    auto_dismiss=False,
                    buttons=[
                        MDFlatButton(
                            text="CANCEL",
                            font_size="18sp",
                            on_press=self.dismiss,
                            font_name=os.path.join(
                                naruno.gui.
                                the_naruno_gui_app.
                                the_naruno_gui.FONT_PATH,
                                "Poppins-Bold",
                            ),
                        ),
                        MDFlatButton(
                            text="OK",
                            font_size="18sp",
                            font_name=os.path.join(
                                naruno.gui.
                                the_naruno_gui_app.
                                the_naruno_gui.FONT_PATH,
                                "Poppins-Bold",
                            ),
                            on_press=self.director,
                        ),
                    ],
                )

                for i in self.inputs:
                    content = i[0]
                    is_pass = i[1]
                    self.dialog.content_cls.add_widget(
                        MDTextField(hint_text=content,
                                    mode="fill",
                                    password=is_pass))
            elif self.type != "question":
                the_type = None if self.type == "qr" else self.type
                self.dialog = SweetAlert()
                self.dialog.fire(
                    self.title,
                    self.text,
                    self.thirdly_title,
                    image=self.image,
                    height_image=self.height_image,
                    type=the_type,
                )
            else:
                self.dialog = SweetAlert()
                self.dialog.fire(
                    title=self.title,
                    text=self.text,
                    type=self.type,
                    buttons=[
                        MDFlatButton(
                            text="NO",
                            font_size="18sp",
                            on_press=self.dismiss,
                            font_name=os.path.join(
                                naruno.gui.
                                the_naruno_gui_app.
                                the_naruno_gui.FONT_PATH,
                                "Poppins-Bold",
                            ),
                        ),
                        MDFlatButton(
                            text="YES",
                            font_size="18sp",
                            font_name=os.path.join(
                                naruno.gui.
                                the_naruno_gui_app.
                                the_naruno_gui.FONT_PATH,
                                "Poppins-Bold",
                            ),
                            on_press=self.director_without_input,
                        ),
                    ],
                )



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os
import zipfile
from hashlib import sha256
from zipfile import ZipFile

from naruno.accounts.get_accounts import GetAccounts
from naruno.blockchain.block.blocks_hash import GetBlockshash
from naruno.blockchain.block.blocks_hash import GetBlockshash_part
from naruno.blockchain.block.get_block import GetBlock
from naruno.blockchain.block.get_block_from_blockchain_db import \
    GetBlockstoBlockchainDB
from naruno.blockchain.block.hash.calculate_hash import CalculateHash
from naruno.config import BLOCKS_PATH
from naruno.config import EXTRACTED_PROOFS_PATH
from naruno.config import MY_TRANSACTION_PATH
from naruno.config import PROOF_PATH
from naruno.lib.config_system import get_config
from naruno.lib.mix.merkle_root import MerkleTree
from naruno.transactions.transaction import Transaction


def CheckProof(
    proof,
    custom_TEMP_BLOCKSHASH_PART_PATH=None,
) -> bool:
    os.chdir(get_config()["main_folder"])

    splitted_proof = proof.split(".")
    temp = splitted_proof[0]
    the_proof_path = EXTRACTED_PROOFS_PATH + temp

    # Create a folder for the proof if it doesn't exist
    if not os.path.exists(the_proof_path):
        # Create the folder
        os.makedirs(the_proof_path)

    try:
        zip_file = zipfile.ZipFile(proof, "r")
    except FileNotFoundError:
        return None
    zip_file.extractall(the_proof_path)
    zip_file.close()

    the_proof_path = the_proof_path + "/" + "db/"
    for file in os.listdir(the_proof_path):
        print(the_proof_path + file)
        if os.path.isdir(the_proof_path + file):
            the_proof_path = the_proof_path + file + "/"

    print(the_proof_path)
    full_blockshash_path = None
    sequence_number = None
    for file in os.listdir(the_proof_path):
        if file.endswith(".blockshash_full.json"):
            full_blockshash_path = the_proof_path + file
        if file.endswith(".block.json"):
            splitted_name = file.split(".")
            sequence_number = splitted_name[0]

    result_2 = GetBlockstoBlockchainDB(
        sequence_number=sequence_number,
        custom_BLOCKS_PATH=the_proof_path,
    )

    Saved_blocks_hash = GetBlockshash(
        custom_TEMP_BLOCKSHASH_PATH=full_blockshash_path)

    hash_2 = CalculateHash(
        result_2[0],
        result_2[3],
        result_2[2],
        result_2[1],
    )

    the_hash_part = MerkleTree([Saved_blocks_hash[0], hash_2]).getRootHash()

    the_blockshash_part = GetBlockshash_part(
        custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH)

    is_in = False
    for i in the_blockshash_part:
        if i == the_hash_part:
            is_in = True
    return is_in



#!/usr/bin/python3
# -*- coding: utf-8 -*-
from base64 import b64decode
from base64 import b64encode

from naruno.wallet.ellipticcurve.utils.compatibility import safeBinaryFromHex
from naruno.wallet.ellipticcurve.utils.compatibility import safeHexFromBinary
from naruno.wallet.ellipticcurve.utils.compatibility import toString


def hexFromInt(number):
    hexadecimal = "{0:x}".format(number)
    if len(hexadecimal) % 2 == 1:
        hexadecimal = f"0{hexadecimal}"
    return hexadecimal


def intFromHex(hexadecimal):
    return int(hexadecimal, 16)


def hexFromByteString(byteString):
    return safeHexFromBinary(byteString)


def byteStringFromHex(hexadecimal):
    return safeBinaryFromHex(hexadecimal)


def numberFromByteString(byteString):
    return intFromHex(hexFromByteString(byteString))


def base64FromByteString(byteString):
    return toString(b64encode(byteString))


def byteStringFromBase64(base64String):
    return b64decode(base64String)


def bitsFromHex(hexadecimal):
    return format(intFromHex(hexadecimal), "b").zfill(4 * len(hexadecimal))



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from naruno.blockchain.block.block_main import Block
from naruno.lib.mix.merkle_root import MerkleTree


def BlocksHash(block: Block, part_of_blocks_hash, the_blocks_hash):
    """
    Calculates and returns the hash of the blocks.
    """

    blocks_hash_list = [block.part_amount_cache]
    blocks_hash_list.extend(iter(the_blocks_hash))
    return MerkleTree(blocks_hash_list).getRootHash()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import time

from naruno.blockchain.block.block_main import Block
from naruno.blockchain.block.get_block import GetBlock
from naruno.lib.settings_system import save_settings
from naruno.lib.settings_system import the_settings
from naruno.node.unl import Unl
from naruno.transactions.my_transactions.get_my_transaction import \
    GetMyTransaction


def Status(
    custom_TEMP_BLOCK_PATH: str = None,
    custom_UNL_NODES_PATH: str = None,
    custom_first_block: Block = None,
    custom_new_block: Block = None,
    custom_connections: list = None,
    custom_transactions: list = None,
    no_cache: bool = False,
    cache_time: int = 300,
    wait_time=50,
) -> dict:
    """
    Returns the status of the network.
    """
    a_settings = the_settings()
    currently_time = time.time()

    if (no_cache or a_settings["status_cache_time"] + cache_time <=
            currently_time) and (not a_settings["status_working"]
                                 or a_settings["status_cache_time"] +
                                 wait_time + 2 <= currently_time):
        a_settings["status_working"] = True
        if not no_cache:
            a_settings["status_cache_time"] = time.time()
        save_settings(a_settings)
        first_block = (GetBlock(custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)
                       if custom_first_block is None else custom_first_block)

        time.sleep(wait_time)
        new_block = (GetBlock(custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)
                     if custom_new_block is None else custom_new_block)

        connections = (Unl.get_as_node_type(
            Unl.get_unl_nodes(custom_UNL_NODES_PATH=custom_UNL_NODES_PATH))
                       if custom_connections is None else custom_connections)
        connected_nodes = [
            str(f"{the_connections.host}:{the_connections.port}")
            for the_connections in connections
        ]

        transactions = (GetMyTransaction() if custom_transactions is None else
                        custom_transactions)


        last_transaction_of_block = (
            str(new_block.validating_list[-1].dump_json())
            if len(new_block.validating_list) > 0 else "")

        status_json = {
            "status": "",
            "first_block": str(first_block.dump_json()),
            "new_block": str(new_block.dump_json()),
            "last_transaction_of_block": last_transaction_of_block,
            "connected_nodes": connected_nodes,
        }

        status_json["status"] = ("Not working" if
                                 (first_block.sequence_number +
                                  first_block.empty_block_number)
                                 == (new_block.sequence_number +
                                     new_block.empty_block_number) else
                                 "Working")

        if not no_cache:
            a_settings["status_cache"] = status_json
            a_settings["status_cache_time"] = time.time()

        a_settings["status_working"] = False
        save_settings(a_settings)
        return status_json

    else:
        return a_settings["status_cache"]



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from naruno.transactions.my_transactions.get_my_transaction import (
    GetMyTransaction,
)


def PrintTransactions():
    """
    Prints all transactions in my transaction.
    """

    print("\n")
    print(
        *[f"{str(i[0].__dict__)} | {str(i[1])}" for i in GetMyTransaction()], sep="\n\n"
    )

    print("\n")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import contextlib

from naruno.blockchain.block.block_main import Block
from naruno.lib.log import get_logger
from naruno.node.server.server import server
from naruno.transactions.cleaner import Cleaner
from naruno.transactions.pending.delete_pending import DeletePending
from naruno.transactions.pending.get_pending import GetPending

logger = get_logger("TRANSACTIONS")


def PendingtoValidating(block: Block):
    """
    Adds transactions to the verification list
    if there are suitable conditions.
    """
    logger.info("Pending to validating transfer process is started")
    first_validating_list_len = len(block.validating_list)
    first_max_tx_number = block.max_tx_number
    logger.debug(f"Currently tx amount: {first_validating_list_len}")
    logger.debug(f"Validating list capacity: {first_max_tx_number}")

    pending_list_txs = GetPending()


    with contextlib.suppress(Exception):
        [
            server.send_transaction(i)
            for i in pending_list_txs + block.validating_list
        ]

    if len(block.validating_list) < block.max_tx_number:
        for tx in OrderbyFee(pending_list_txs):
            if len(block.validating_list) < block.max_tx_number:
                logger.info(f"tx {tx.signature} is moved to validating list")

                block.validating_list.append(tx)

                DeletePending(tx)
            else:
                logger.info(
                    f"TX {tx.signature} is can not moved to validating list")
    else:
        logger.info("List is full")


def OrderbyFee(transactions: list):
    """
    Sorts transactions by fee.
    """

    transactions.sort(key=lambda x: x.transaction_fee, reverse=True)
    return transactions



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os
from hashlib import sha256

from naruno.config import *
from naruno.lib.config_system import get_config
from naruno.wallet.get_saved_wallet import get_saved_wallet


def save_to_wallet_list(publicKey, privateKey, password):
    wallet_list = get_saved_wallet()

    wallet_list[publicKey] = {}

    wallet_list[publicKey]["publickey"] = publicKey.replace("\n", "")
    wallet_list[publicKey]["privatekey"] = privateKey

    wallet_list[publicKey]["password_sha256"] = sha256(
        password.encode("utf-8")).hexdigest()

    save_wallet_list(wallet_list)


def save_wallet_list(wallet_list):

    os.chdir(get_config()["main_folder"])
    with open(WALLETS_PATH, "w") as wallet_list_file:
        json.dump(wallet_list, wallet_list_file, indent=4)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import sqlite3

from naruno.accounts.account import Account
from naruno.accounts.save_accounts import SaveAccounts
from naruno.blockchain.block.shares import shares
from naruno.config import TEMP_ACCOUNTS_PATH
from naruno.consensus.rounds.round_1.process.transactions.checks.duplicated import \
    Remove_Duplicates
from naruno.lib.log import get_logger
from naruno.wallet.wallet_import import Address

logger = get_logger("TRANSACTIONS")


def ProccesstheTransaction(
    block,
    the_account_list,
    custom_TEMP_ACCOUNTS_PATH=None,
    custom_shares=None,
    custom_fee_address=None,
    dont_clean=False,
):
    """
    It performs the transactions in the block.vali list and
    puts the transactions in order.

    Queuing is required so that all nodes have the same transaction hash.
    """

    logger.info("Transaction processing started.")

    if not dont_clean:
        block = Remove_Duplicates(block)

    the_TEMP_ACCOUNTS_PATH = (TEMP_ACCOUNTS_PATH
                              if custom_TEMP_ACCOUNTS_PATH is None else
                              custom_TEMP_ACCOUNTS_PATH)

    edited_accounts = []

    clean_list = []
    for unclear in block.validating_list:
        if not "NARUNO" in unclear.signature:
            clean_list.append(unclear)
    block.validating_list = clean_list

    the_shares = shares(
        block,
        custom_shares=custom_shares,
        custom_fee_address=custom_fee_address,
        dont_clean=dont_clean,
    )
    block.validating_list = block.validating_list + the_shares

    new_added_accounts_list = []
    account_list = []

    block.validating_list = sorted(block.validating_list,
                                   key=lambda x: x.fromUser)

    temp_validating_list = block.validating_list

    actions = []

    for trans in block.validating_list:
        logger.info(f"Transaction: {trans.__dict__}")

        touser_inlist = True
        to_user_in_new_list = False

        address_of_fromUser = Address(trans.fromUser)
        logger.debug(f"FromUser address: {address_of_fromUser}")
        the_account_list.execute(
            f"SELECT * FROM account_list WHERE address = '{address_of_fromUser}'"
        )
        first_list = the_account_list.fetchall()
        the_account_list.execute(
            f"SELECT * FROM account_list WHERE address = '{trans.toUser}'")
        second_list = the_account_list.fetchall()

        for the_pulled_account in first_list + second_list:
            account_list.append(
                Account(the_pulled_account[0], the_pulled_account[2],
                        the_pulled_account[1]))

        for Accounts in account_list:
            touser_inlist = False

            if Accounts.Address == address_of_fromUser:
                logger.debug(f"FromUser found: {Accounts.Address}")
                actions.append([
                    Accounts.Address,
                    "balance",
                    -(float(trans.amount) + trans.transaction_fee),
                ])
                actions.append([Accounts.Address, "sequence_number", 1])

            if Accounts.Address == trans.toUser:
                logger.debug(f"ToUser found: {Accounts.Address}")
                actions.append(
                    [Accounts.Address, "balance",
                     float(trans.amount)])
                touser_inlist = True

        for i in new_added_accounts_list:
            if i.Address == trans.toUser:
                i.balance += float(trans.amount)
                to_user_in_new_list = True

        # If not included in the account_list, add.
        if not touser_inlist and not to_user_in_new_list:
            new_added_accounts_list.append(
                Account(trans.toUser, float(trans.amount)))

    for action in actions:
        for account in account_list:
            if action[0] == account.Address:
                if action[1] == "balance":
                    account.balance += action[2]
                elif action[1] == "sequence_number":
                    account.sequence_number += action[2]
                edited_accounts.append(account)

    # Syncs new sorted list to block.validating_list

    block.validating_list = sorted(temp_validating_list,
                                   key=lambda x: x.signature)

    new_added_accounts_list = sorted(new_added_accounts_list,
                                     key=lambda x: x.Address)

    conn = sqlite3.connect(the_TEMP_ACCOUNTS_PATH)
    c = conn.cursor()
    for changed_account in edited_accounts:
        c.execute(
            f"UPDATE account_list SET balance = {changed_account.balance}, sequence_number = {changed_account.sequence_number} WHERE address = '{changed_account.Address}'"
        )
        conn.commit()
    conn.close()

    SaveAccounts(new_added_accounts_list, the_TEMP_ACCOUNTS_PATH)

    return block



#!/usr/bin/python3
# -*- coding: utf-8 -*-
from random import SystemRandom


class RandomInteger:

    @classmethod
    def between(cls, min, max):
        """
        Return integer x in the range: min <= x <= max

        :param min: minimum value of the integer
        :param max: maximum value of the integer
        :return:
        """

        return SystemRandom().randrange(min, max + 1)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os
from hashlib import sha256

from naruno.config import COMMANDERS_PATH
from naruno.lib.config_system import get_config


def SaveCommander(commander):
    the_path = COMMANDERS_PATH + commander
    os.chdir(get_config()["main_folder"])
    with open(the_path, "w") as my_transaction_file:
        my_transaction_file.write("1")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from naruno.blockchain.block.block_main import Block
from naruno.blockchain.block.save_block import SaveBlock
from naruno.blockchain.candidate_block.candidate_block_main import \
    candidate_block
from naruno.consensus.rounds.round_1.round_1_main import \
    consensus_round_1
from naruno.consensus.rounds.round_2.round_2_main import \
    consensus_round_2
from naruno.lib.log import get_logger
from naruno.node.client.client import client
from naruno.node.server.server import server
from naruno.transactions.cleaner import Cleaner
from naruno.transactions.pending.get_pending import GetPending
logger = get_logger("CONSENSUS")


def ongoing_main(
    block: Block,
    custom_candidate_class: candidate_block = None,
    custom_unl_nodes: dict = None,
    custom_UNL_NODES_PATH: str = None,
    custom_server: server = None,
    custom_unl: client = None,
    custom_TEMP_ACCOUNTS_PATH: str = None,
    custom_TEMP_BLOCK_PATH: str = None,
    custom_TEMP_BLOCKSHASH_PATH: str = None,
    custom_TEMP_BLOCKSHASH_PART_PATH: str = None,
    custom_shares=None,
    custom_fee_address=None,
    pass_sync=False,
    clean=True
) -> Block:

    block.sync_empty_blocks() if pass_sync is False else None
    SaveBlock(
            block,
            custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH,
            custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
            custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH,
            custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH,
        )    

    if not block.round_1:    
        logger.debug("First round is starting")
        if clean:
            cleaned = Cleaner(block, pending_list_txs=GetPending())
            block.validating_list = cleaned[0]      
        consensus_round_1(
            block,
            custom_candidate_class=custom_candidate_class,
            custom_unl_nodes=custom_unl_nodes,
            custom_UNL_NODES_PATH=custom_UNL_NODES_PATH,
            custom_server=custom_server,
            custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
            custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH,
            custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH,
            custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH,
            custom_shares=custom_shares,
            custom_fee_address=custom_fee_address,
            clean=clean
        )
        logger.debug("First round is done")
    elif not block.round_2:
        logger.debug("Second round is starting")
        consensus_round_2(
            block,
            candidate_class=custom_candidate_class,
            unl_nodes=custom_unl_nodes,
            custom_server=custom_server,
            custom_unl=custom_unl,
            custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH,
            custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
            custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH,
            custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH,
        )
        logger.debug("Second round is done")
    return block



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import contextlib
import hashlib
import os
import sys
import time

from speed_calculator import calculate
sys.path.append(os.path.join(os.path.dirname(__file__), "..", "..", ".."))
from naruno.lib.mix.merkle_root import MerkleTree



from naruno.accounts.account import Account
from naruno.accounts.get_accounts import GetAccounts
from naruno.accounts.save_accounts import SaveAccounts
from naruno.blockchain.block.block_main import Block
from naruno.blockchain.block.blocks_hash import (GetBlockshash,
                                                           GetBlockshash_part,
                                                           SaveBlockshash,
                                                           SaveBlockshash_part)
from naruno.blockchain.block.get_block import GetBlock
from naruno.blockchain.block.save_block import SaveBlock


class Blockshash_part_IO_Performance_Analyzer:
    """
    This class is used to analyze the performance of GetBlock
    """

    def __init__(self):
        self.block = Block("test")
        self.the_hash = hashlib.sha256("test".encode()).hexdigest()

        # calculate the seconds in a day
        seconds_in_a_day = 60 * 60 * 24

        # how many blocks in a day
        blocks_in_a_day = seconds_in_a_day / self.block.block_time

        # how many blocks in a year
        blocks_in_a_year = (blocks_in_a_day * 365) // self.block.part_amount
        self.blocks_hash = [
            self.the_hash for i in range(int(blocks_in_a_year))
        ]

        SaveBlockshash_part(
            self.blocks_hash,
            custom_TEMP_BLOCKSHASH_PART_PATH=
            "db/Blockshash_part_Performance_Analyzer_blockshash.pf",
        )

    def analyze(self) -> float:
        """
        This function is used to analyze the performance of GetBlock
        """

        result = (
            calculate(self.save_operation)[0],
            calculate(self.get_operation)[0],
            os.path.getsize(
                "db/Blockshash_part_Performance_Analyzer_blockshash.pf") /
            1000000,
        )

        os.remove("db/Blockshash_part_Performance_Analyzer_blockshash.pf")

        return result

    def save_operation(self):
        """
        This function is used to analyze the performance of GetBlock
        """

        SaveBlockshash_part(
            self.the_hash,
            custom_TEMP_BLOCKSHASH_PART_PATH=
            "db/Blockshash_part_Performance_Analyzer_blockshash.pf",
        )

        MerkleTree(self.blocks_hash).getRootHash()

    def get_operation(self):
        """
        This function is used to analyze the performance of GetBlock
        """

        GetBlockshash_part(
            custom_TEMP_BLOCKSHASH_PART_PATH=
            "db/Blockshash_part_Performance_Analyzer_blockshash.pf")


if __name__ == "__main__":
    print(Blockshash_part_IO_Performance_Analyzer().analyze())



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import copy
import json
import os
import time

from naruno.accounts.account import Account
from naruno.accounts.get_accounts import GetAccounts
from naruno.accounts.save_accounts import SaveAccounts
from naruno.blockchain.block.blocks_hash import SaveBlockshash
from naruno.config import TEMP_BLOCK_PATH
from naruno.lib.config_system import get_config
from naruno.lib.log import clear_logs
from naruno.lib.log import get_logger
from naruno.lib.settings_system import the_settings
from naruno.node.unl import Unl
from naruno.transactions.transaction import Transaction

logger = get_logger("BLOCKCHAIN")


class Block:
    """
    Block class is most important class. It is responsible for
    resetting and saving blocks.

    You must give a creator of the block. This creator will
    own all the coins.
    """

    def __init__(
        self,
        creator,
        previous_hash="1a00d983803e3adcbda2ed40ecba828083221648a90150267d8b0fd500c59750",
    ):
        self.coin_amount = 10000000
        self.first_time = True
        self.creator = creator
        self.genesis_time = int(time.time())
        self.start_time = int(time.time())
        self.block_time = 27

        self.previous_hash = previous_hash
        self.sequence_number = 0
        self.empty_block_number = 0
        self.hard_block_number = 2
        self.gap_block_number = self.hard_block_number + 2

        self.validating_list = []
        self.transaction_fee = 0.02
        self.default_transaction_fee = 0.02
        self.default_optimum_transaction_number = 200
        self.default_increase_of_fee = 0.01
        self.transaction_delay_time = 3600
        self.max_data_size = 1000000

        self.part_amount = 1000

        self.hash = None
        self.part_amount_cache = previous_hash

        self.max_tx_number = 1000
        self.minumum_transfer_amount = 1000

        self.round_1_time = 20
        self.round_1 = False

        self.round_2_starting_time = None
        self.round_2_time = 5
        self.round_2 = False

        self.consensus_timer = 0.50

        self.validated = False
        self.validated_time = None

        self.dowload_true_block = ""
        self.sync = False

        self.just_one_tx = True
        self.use_full_block = True

        self.shares = []
        self.fee_address = creator

        if the_settings()["funtionaltest_mode"]:
            self.max_tx_number = 2
            self.max_data_size = 1000

    def reset_the_block(self, custom_nodes=None):
        """
        When the block is verified and if block have a transaction
        and if block have at least half of the max_tx_number transaction,it saves the block
        and makes the edits for the new block.
        """

        self.start_time = (self.genesis_time +
                           ((self.sequence_number + self.empty_block_number) *
                            self.block_time)) + self.block_time

        self.round_1 = False

        self.round_2_starting_time = None
        self.round_2 = False

        self.validated = False
        self.validated_time = None

        if (not self.use_full_block and len(self.validating_list) != 0) or len(
                self.validating_list) >= (self.max_tx_number / 2):
            block2 = copy.copy(self)
            # Resetting and setting the new elements.
            self.previous_hash = self.hash
            self.sequence_number = self.sequence_number + 1
            clear_logs()
            self.validating_list = []
            self.hash = None
            logger.info("New block created")
            self.sync_empty_blocks()
            logger.debug(self.__dict__)
            return [block2, self]
        else:
            logger.info(
                "New block not created because no transaction enought to create a new block"
            )
            self.sync_empty_blocks()
            return False

    def sync_empty_blocks(self):
        if not self.round_2 and not self.round_1:
            first_empty_block = self.empty_block_number
            sequence_number_time = self.genesis_time + (
                (self.sequence_number) * self.block_time)
            extra = int(time.time()) - sequence_number_time
            adding = extra // self.block_time
            secondly_empty_block = adding
            if not first_empty_block > secondly_empty_block:
                if not first_empty_block == secondly_empty_block:
                    clear_logs()
                self.empty_block_number = adding
                self.start_time = self.genesis_time + (
                    (self.sequence_number + self.empty_block_number) *
                    self.block_time)
                if self.round_1:
                    self.round_2_starting_time = self.start_time + self.round_1_time
            else:
                self.start_time = self.genesis_time + (
                    (self.sequence_number + self.empty_block_number -
                     self.gap_block_number) * self.block_time)

    def dump_json(self):
        """
        Dumps the block as json.
        """
        temp_block = copy.copy(self)

        temp_validating_list = [
            transaction.dump_json()
            for transaction in temp_block.validating_list
        ]

        temp_block.validating_list = temp_validating_list
        return temp_block.__dict__

    @staticmethod
    def load_json(json_string):
        temp_validating_list = [
            Transaction.load_json(tx) for tx in json_string["validating_list"]
        ]

        the_block_json = json.loads(json.dumps(json_string))
        the_block_json["validating_list"] = temp_validating_list
        the_block = Block("Naruno")
        the_block.__dict__ = the_block_json

        return the_block



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import argparse
import contextlib
import os
import sys

from flask import Flask
from flask import jsonify
from flask import request
from waitress import serve
from waitress.server import create_server

sys.path.append(os.path.join(os.path.dirname(__file__), "..", ".."))

from flask_cors import CORS

from naruno.accounts.get_balance import GetBalance
from naruno.accounts.get_sequence_number import GetSequanceNumber
from naruno.blockchain.block.create_block import CreateBlock
from naruno.blockchain.block.get_block import GetBlock
from naruno.blockchain.block.just_one_tx import GetJustOneTX
from naruno.blockchain.block.max_data_size import GetMaxDataSize
from naruno.blockchain.block.max_tx_number import GetMaxTXNumber
from naruno.blockchain.block.save_block import SaveBlock
from naruno.consensus.consensus_main import consensus_trigger
from naruno.lib.export import export_the_transactions
from naruno.lib.log import get_logger
from naruno.lib.perpetualtimer import perpetualTimer
from naruno.lib.safety import safety_check
from naruno.lib.settings_system import (d_mode_settings, ft_mode_settings,
                                        t_mode_settings, the_settings)
from naruno.lib.sign import sign
from naruno.lib.status import Status
from naruno.lib.verify import verify
from naruno.node.server.server import server
from naruno.node.unl import Unl
from naruno.transactions.get_transaction import GetTransaction
from naruno.transactions.my_transactions.check_proof import CheckProof
from naruno.transactions.my_transactions.get_my_transaction import \
    GetMyTransaction
from naruno.transactions.my_transactions.get_proof import GetProof
from naruno.transactions.my_transactions.save_to_my_transaction import \
    SavetoMyTransaction
from naruno.transactions.send import send
from naruno.transactions.transaction import Transaction
from naruno.wallet.delete_current_wallet import delete_current_wallet
from naruno.wallet.print_wallets import print_wallets
from naruno.wallet.wallet_create import wallet_create
from naruno.wallet.wallet_import import wallet_import
from naruno.wallet.wallet_selector import wallet_selector

logger = get_logger("API")

app = Flask(__name__)
CORS(app,
     resources={
         r"/export/block/*": {
             "origins": "*"
         },
         r"/status": {
             "origins": "*"
         }
     })

custom_block = None
custom_current_time = None
custom_sequence_number = None
custom_balance = None
custom_server = None

custom_TEMP_BLOCK_PATH = None
custom_TEMP_ACCOUNTS_PATH = None
custom_TEMP_BLOCKSHASH_PATH = None
custom_TEMP_BLOCKSHASH_PART_PATH = None

account_list = None

custom_wallet = None

custom_CONNECTED_NODES_PATH = None

custom_consensus_trigger = None
custom_consensus_trigger_result = None

custom_transactions = None
custom_MY_TRANSACTION_EXPORT_PATH = None

custom_UNL_NODES_PATH = None
custom_first_block = None
custom_new_block = None
custom_connections = None

custom_account_list = None


@app.route("/wallet/print", methods=["GET"])
def print_wallets_page():
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    if the_settings()["publisher_mode"]:
        return jsonify(
            {"error": "You can't print the wallets in publisher mode."})
    return jsonify(print_wallets())


@app.route("/wallet/change/<number>", methods=["GET"])
def wallet_change_page(number):
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    if the_settings()["publisher_mode"]:
        return jsonify(
            {"error": "You can't change the wallet in publisher mode."})
    wallet_selector(number)
    return jsonify(print_wallets())


@app.route("/wallet/create/<password>", methods=["GET"])
def create_wallet_page(password):
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    if the_settings()["publisher_mode"]:
        return jsonify(
            {"error": "You can't create a wallet in publisher mode."})
    wallet_create(password)
    return jsonify(print_wallets())


@app.route("/wallet/delete", methods=["GET"])
def delete_wallets_page():
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    if the_settings()["publisher_mode"]:
        return jsonify(
            {"error": "You can't delete the wallet in publisher mode."})
    delete_current_wallet()
    return jsonify(print_wallets())


@app.route("/send/", methods=["POST"])
def send_coin_data_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.form}")
    try:
        if the_settings()["publisher_mode"]:
            return jsonify(
                {"error": "You can't send a transaction in publisher mode."})
        address = str(
            request.form["to_user"]) if "to_user" in request.form else None
        amount = float(
            request.form["amount"]) if "amount" in request.form else None
        data = str(request.form["data"]) if "data" in request.form else ""
        password = str(
            request.form["password"]) if "password" in request.form else None
        sequence_number = (str(request.form["sequence_number"])
                           if "sequence_number" in request.form else None)
        block = None
        with contextlib.suppress(Exception):
            block = (GetBlock(custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)
                     if custom_block is None else custom_block)
        send_tx = send(
            password,
            address,
            amount=amount,
            data=data,
            block=block,
            custom_current_time=custom_current_time,
            custom_sequence_number=custom_sequence_number,
            custom_balance=custom_balance,
            custom_account_list=custom_account_list,
            custom_set_sequence_number=sequence_number,
        )
    except:
        result = "false"
    if send_tx != False:
        SavetoMyTransaction(send_tx, sended=True)
        if not the_settings()["baklava"]:
            server.send_transaction(
                send_tx,
                custom_current_time=custom_current_time,
                custom_sequence_number=custom_sequence_number,
                custom_balance=custom_balance,
                custom_server=custom_server,
            )
            SaveBlock(
                block,
                custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH,
                custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
                custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH,
                custom_TEMP_BLOCKSHASH_PART_PATH=
                custom_TEMP_BLOCKSHASH_PART_PATH,
            )
    result = send_tx.dump_json() if send_tx != False else False

    return jsonify(result)


@app.route("/wallet/balance", methods=["GET"])
def balance_wallets_page():
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    if the_settings()["publisher_mode"]:
        return jsonify(
            {"error": "You can't get the balance in publisher mode."})
    the_wallet = wallet_import(-1,
                               0) if custom_wallet is None else custom_wallet
    the_block = (GetBlock(custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)
                 if custom_block is None else custom_block)
    return jsonify(
        GetBalance(the_wallet, account_list=account_list, block=the_block))


@app.route("/node/start/<ip>/<port>", methods=["GET"])
def node_start_page(ip, port):
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    if the_settings()["publisher_mode"]:
        return jsonify(
            {"error": "You can't start the node in publisher mode."})
    server(str(ip), int(port))
    return jsonify("OK")


@app.route("/node/stop", methods=["GET"])
def node_stop_page():
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    if the_settings()["publisher_mode"]:
        return jsonify({"error": "You can't stop the node in publisher mode."})
    server.Server.stop()
    return jsonify("OK")


@app.route("/node/connect/<ip>/<port>", methods=["GET"])
def node_connect_page(ip, port):
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    if the_settings()["publisher_mode"]:
        return jsonify(
            {"error": "You can't connect to the node in publisher mode."})
    server.Server.connect(str(ip), int(port))
    return jsonify("OK")


@app.route("/node/connectmixdb", methods=["GET"])
def node_connectmixdb_page():
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    if the_settings()["publisher_mode"]:
        return jsonify(
            {"error": "You can't connect to the node in publisher mode."})
    server.connectionfrommixdb(
        custom_server=custom_server,
        custom_CONNECTED_NODES_PATH=custom_CONNECTED_NODES_PATH,
    )
    return jsonify("OK")


# /node/newunl/?MFYw......
@app.route("/node/newunl/", methods=["GET"])
def node_newunl_page():
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    if the_settings()["publisher_mode"]:
        return jsonify(
            {"error": "You can't connect to the node in publisher mode."})
    Unl.save_new_unl_node(request.query_string.decode("utf-8"))
    return jsonify("OK")


@app.route("/node/id", methods=["GET"])
def node_id_page():
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    if the_settings()["publisher_mode"]:
        return jsonify(
            {"error": "You can't get the node id in publisher mode."})
    return jsonify(server.id)


@app.route("/settings/test/on", methods=["GET"])
def settings_test_on_page():
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    if the_settings()["publisher_mode"]:
        return jsonify(
            {"error": "You can't turn on the test mode in publisher mode."})
    t_mode_settings(True)
    return jsonify("OK")


@app.route("/settings/test/off", methods=["GET"])
def settings_test_off_page():
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    if the_settings()["publisher_mode"]:
        return jsonify(
            {"error": "You can't turn off the test mode in publisher mode."})
    t_mode_settings(False)
    return jsonify("OK")


@app.route("/settings/debug/on", methods=["GET"])
def settings_debug_on_page():
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    if the_settings()["publisher_mode"]:
        return jsonify(
            {"error": "You can't turn on the debug mode in publisher mode."})
    app.config["DEBUG"] = True
    d_mode_settings(True)
    return jsonify("OK")


@app.route("/settings/debug/off", methods=["GET"])
def settings_debug_off_page():
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    if the_settings()["publisher_mode"]:
        return jsonify(
            {"error": "You can't turn off the debug mode in publisher mode."})
    app.config["DEBUG"] = False
    d_mode_settings(False)
    return jsonify("OK")


@app.route("/settings/functionaltest/on", methods=["GET"])
def fsettings_debug_on_page():
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    if the_settings()["publisher_mode"]:
        return jsonify(
            {"error": "You can't turn on the debug mode in publisher mode."})
    app.config["DEBUG"] = True
    ft_mode_settings(True)
    return jsonify("OK")


@app.route("/settings/functionaltest/off", methods=["GET"])
def fsettings_debug_off_page():
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    if the_settings()["publisher_mode"]:
        return jsonify(
            {"error": "You can't turn off the debug mode in publisher mode."})
    app.config["DEBUG"] = False
    ft_mode_settings(False)
    return jsonify("OK")


@app.route("/block/get", methods=["GET"])
def block_get_page():
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    if the_settings()["publisher_mode"]:
        return jsonify({"error": "You can't get the block in publisher mode."})
    the_server = server.Server if custom_server is None else custom_server
    if the_settings()["test_mode"]:
        the_block = CreateBlock(custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)
        SaveBlock(
            the_block,
            custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH,
            custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
            custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH,
            custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH,
        )
        the_server.send_block_to_other_nodes()
        logger.info("Consensus timer is started")
        the_consensus_trigger = (consensus_trigger
                                 if custom_consensus_trigger is None else
                                 custom_consensus_trigger)
        trigger = perpetualTimer(the_block.consensus_timer,
                                 the_consensus_trigger)
        global custom_consensus_trigger_result
        custom_consensus_trigger_result = trigger
    else:
        the_server.send_me_full_block()
    return jsonify("OK")


@app.route("/export/transactions/csv", methods=["GET"])
def export_transaction_csv_page():
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")

    return jsonify(
        export_the_transactions(
            custom_transactions=custom_transactions,
            custom_MY_TRANSACTION_EXPORT_PATH=custom_MY_TRANSACTION_EXPORT_PATH,
        ))


@app.route("/transactions/sended/validated", methods=["GET"])
def transaction_sended_validated_page():
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    return jsonify(
        GetMyTransaction(sended=True, validated=True, turn_json=True))


@app.route("/transactions/sended/not_validated", methods=["GET"])
def transaction_sended_not_validated_page():
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    return jsonify(
        GetMyTransaction(sended=True, validated=False, turn_json=True))


@app.route("/transactions/received", methods=["GET"])
def transaction_received_page():
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    return jsonify(GetMyTransaction(sended=False, turn_json=True))


@app.route("/transactions/all", methods=["GET"])
def transaction_all_page():
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    return jsonify(GetMyTransaction(turn_json=True))


@app.route("/status", methods=["GET"])
def status_page():
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    return jsonify(
        Status(
            custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH,
            custom_UNL_NODES_PATH=custom_UNL_NODES_PATH,
            custom_first_block=custom_first_block,
            custom_new_block=custom_new_block,
            custom_connections=custom_connections,
            custom_transactions=custom_transactions,
        ))


@app.route("/proof/get/", methods=["POST"])
def proof_get_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.form}")
    if the_settings()["publisher_mode"]:
        return jsonify({"error": "You can't get the proof in publisher mode."})
    signature = str(
        request.form["signature"]) if "signature" in request.form else None
    custom_PROOF_PATH = (str(request.form["custom_PROOF_PATH"])
                         if "custom_PROOF_PATH" in request.form else None)
    custom_BLOCKS_PATH = (str(request.form["custom_BLOCKS_PATH"])
                          if "custom_BLOCKS_PATH" in request.form else None)
    custom_TEMP_ACCOUNTS_PATH = (str(request.form["custom_TEMP_ACCOUNTS_PATH"])
                                 if "custom_TEMP_ACCOUNTS_PATH" in request.form
                                 else None)
    custom_TEMP_BLOCKSHASH_PATH = (
        str(request.form["custom_TEMP_BLOCKSHASH_PATH"])
        if "custom_TEMP_BLOCKSHASH_PATH" in request.form else None)
    custom_TEMP_BLOCKSHASH_PART_PATH = (
        str(request.form["custom_TEMP_BLOCKSHASH_PART_PATH"])
        if "custom_TEMP_BLOCKSHASH_PART_PATH" in request.form else None)

    return jsonify(
        GetProof(
            signature,
            custom_PROOF_PATH=custom_PROOF_PATH,
            custom_BLOCKS_PATH=custom_BLOCKS_PATH,
            custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
            custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH,
            custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH,
        ))


@app.route("/proof/check/", methods=["POST"])
def proof_check_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.form}")
    if the_settings()["publisher_mode"]:
        return jsonify(
            {"error": "You can't check the proof in publisher mode."})
    path = str(request.form["path"]) if "path" in request.form else None
    custom_TEMP_BLOCKSHASH_PART_PATH = (
        str(request.form["custom_TEMP_BLOCKSHASH_PART_PATH"])
        if "custom_TEMP_BLOCKSHASH_PART_PATH" in request.form else None)
    return jsonify(
        CheckProof(
            path,
            custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH,
        ))


@app.route("/sign/", methods=["POST"])
def sign_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.form}")
    if the_settings()["publisher_mode"]:
        return jsonify({"error": "You can't sign in publisher mode."})
    data = str(request.form["data"]) if "data" in request.form else None

    password = str(
        request.form["password"]) if "password" in request.form else None

    return jsonify(sign(
        data,
        password,
    ))


@app.route("/verify/", methods=["POST"])
def verify_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.form}")
    if the_settings()["publisher_mode"]:
        return jsonify({"error": "You can't verify in publisher mode."})
    path = str(request.form["path"]) if "path" in request.form else None

    return jsonify(verify(path))


@app.route("/export/block/json", methods=["GET"])
def export_block_json_page():
    logger.debug(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    the_block = (GetBlock(custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)
                 if custom_block is None else custom_block)
    return jsonify(the_block.dump_json())


@app.route("/balance/get/", methods=["GET"])
def balance_get_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.form}")
    # Check publisher mode
    if not the_settings()["publisher_mode"]:
        return jsonify("403"), 403
    address = str(request.args.get("address"))
    the_block = (GetBlock(custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)
                 if custom_block is None else custom_block)

    return jsonify(
        GetBalance(
            address,
            block=the_block,
            account_list=custom_account_list,
            dont_convert=True,
        ))


@app.route("/sequence/get/", methods=["GET"])
def sequence_get_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.form}")
    # Check publisher mode
    if not the_settings()["publisher_mode"]:
        return jsonify("403"), 403
    address = str(request.args.get("address"))

    the_block = (GetBlock(custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)
                 if custom_block is None else custom_block)

    return jsonify(
        GetSequanceNumber(
            address,
            account_list=custom_account_list,
            dont_convert=True,
            block=the_block,
        ))


# Write a api for directing a transaction with GetTransaction
@app.route("/transaction/send/", methods=["POST"])
def transaction_send_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.form}")

    if not the_settings()["publisher_mode"]:
        return jsonify("403"), 403

    sequence_number = int(request.form["sequence_number"])
    signature = str(request.form["signature"])
    fromUser = str(request.form["fromUser"])

    toUser = str(request.form["toUser"])
    amount = float(request.form["amount"])
    data = str(request.form["data"])
    transaction_fee = float(request.form["transaction_fee"])

    time_of_transaction = int(request.form["time_of_transaction"])

    the_transaction = Transaction(
        sequence_number=sequence_number,
        signature=signature,
        fromUser=fromUser,
        toUser=toUser,
        amount=amount,
        data=data,
        transaction_fee=transaction_fee,
        time_of_transaction=time_of_transaction,
    )

    block = (GetBlock(custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)
             if custom_block is None else custom_block)

    if GetTransaction(
            block,
            the_transaction,
            custom_current_time=custom_current_time,
            custom_sequence_number=custom_sequence_number,
            custom_balance=custom_balance,
            custom_account_list=custom_account_list,
    ):
        return jsonify("200"), 200
    else:
        return jsonify("400"), 400


@app.route("/blocktransactionfee/get/", methods=["GET"])
def blocktransactionfee_get_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.form}")
    # Check publisher mode
    if not the_settings()["publisher_mode"]:
        return jsonify("403"), 403
    the_block = (GetBlock(custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)
                 if custom_block is None else custom_block)

    return jsonify(the_block.transaction_fee)


@app.route("/blockmaxtxnumber/get/", methods=["GET"])
def blockmaxtxnumber_get_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.form}")
    # Check publisher mode
    the_block = (GetBlock(custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)
                 if custom_block is None else custom_block)

    return jsonify(GetMaxTXNumber(block=the_block))


@app.route("/blockjustonetx/get/", methods=["GET"])
def blockjustonetx_get_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.form}")
    # Check publisher mode
    the_block = (GetBlock(custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)
                 if custom_block is None else custom_block)

    return jsonify(GetJustOneTX(block=the_block))


@app.route("/blockmaxdatasize/get/", methods=["GET"])
def blockmaxdatasize_get_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.form}")
    # Check publisher mode

    the_block = (GetBlock(custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)
                 if custom_block is None else custom_block)

    return jsonify(GetMaxDataSize(block=the_block))


@app.route("/blockminumumtransferamount/get/", methods=["GET"])
def blockminumumtransferamount_get_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.form}")
    # Check publisher mode
    if not the_settings()["publisher_mode"]:
        return jsonify("403"), 403
    the_block = (GetBlock(custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)
                 if custom_block is None else custom_block)

    return jsonify(the_block.minumum_transfer_amount)


@app.errorhandler(500)
def handle_exception(e):
    logger.exception(f"500: {e}")
    return jsonify("500"), 500


@app.errorhandler(404)
def page_not_found(e):
    logger.error(f"404: {e}")
    return jsonify("404"), 404


@app.errorhandler(405)
def method_not_allowed(e):
    logger.error(f"405: {e}")
    return jsonify("405"), 405


def start(host=None, port=None, test=False):
    """
    Start the API server.
    """
    if host is None:
        host = "0.0.0.0"

    parser = argparse.ArgumentParser(
        description=
        "Naruno is a lightning-fast, secure, and scalable blockchain that is able to create transaction proofs and verification via raw data and timestamp. We remove the archive nodes and lazy web3 integrations. With Naruno everyone can get the proof (5-10MB) of their transactions via their nodes and after everyone can use in another node for verification the raw data and timestamp. Also you can integrate your web3 applications with 4 code lines (just python for now) via our remote app system."
    )

    parser.add_argument("-p",
                        "--port",
                        default=8000,
                        type=int,
                        help="Add new UNL node")

    parser.add_argument(
        "-i",
        "--interface",
        type=str,
        help="Interface",
    )

    parser.add_argument(
        "-t",
        "--timeout",
        type=int,
        help="Timeout",
    )

    args = parser.parse_args()

    args.port = args.port if port is None else int(port)

    safety_check(args.interface, args.timeout)

    logger.info(f"Starting API on port {args.port}")
    result = (serve(app, host=host, port=args.port) if test is False else
              create_server(app, host=host, port=args.port))
    return result


if __name__ == "__main__":
    start()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import contextlib
import copy
import inspect
import json
import math
import os
import random
import string
import sys
import threading
import time
from hashlib import sha256

import requests

from naruno.accounts.commanders.delete_commander import DeleteCommander
from naruno.accounts.commanders.get_comnder import GetCommander
from naruno.accounts.commanders.save_commander import SaveCommander
from naruno.api.main import start
from naruno.apps.checker import checker
from naruno.blockchain.block.block_main import Block
from naruno.lib.config_system import get_config
from naruno.lib.log import get_logger
from naruno.lib.perpetualtimer import perpetualTimer
from naruno.lib.settings_system import baklava_settings
from naruno.lib.settings_system import the_settings
from naruno.transactions.my_transactions.save_to_my_transaction import \
    SavetoMyTransaction
from naruno.transactions.my_transactions.sended_transaction import \
    SendedTransaction
from naruno.transactions.my_transactions.validate_transaction import \
    ValidateTransaction
from naruno.transactions.transaction import Transaction
from naruno.wallet.wallet_import import Address
from naruno.wallet.wallet_import import wallet_import

logger = get_logger("REMOTE_APP")


class splitted_data:

    def __init__(self, split):
        self.split = split
        self.main_data = None
        self.validated = False
        self.data = []
        self.data_original = []


class Integration:

    def __init__(
        self,
        app_name,
        host="localhost",
        port=8000,
        password="123",
        sended=False,
        sended_not_validated=False,
        cache_true=True,
        wait_amount=None,
        checking=True,
        commander=None,
        total_check=None,
    ):
        """
        :param host: The host of the node
        :param port: The port of the node
        :param password: The password of the wallet
        """
        self.app_name = app_name
        self.cache_name = sha256(
            self.app_name.encode()).hexdigest() + wallet_import(-1, 3)
        self.host = host
        self.port = port

        self.first_host = copy.copy(host)
        self.first_port = copy.copy(port)

        self.api = None

        self.init_api()

        self.password = password

        self.sended = sended

        self.sended_not_validated = sended_not_validated

        self.cache_true = cache_true

        self.last_sended = 0

        a_block = Block("Onur")

        if wait_amount is None:
            self.wait_amount = a_block.block_time * 2
        else:
            self.wait_amount = wait_amount

        self.get_cache()

        self.sended_txs = []

        self.checking = checking

        self.commander = commander
        SaveCommander(self.commander) if not self.commander is None else None

        self.check_thread = None

        backup_host = copy.copy(self.host)
        backup_port = copy.copy(self.port)
        if the_settings()["baklava"]:
            self.host = "test_net.1.naruno.org"
            self.port = 8000
        try:
            self.max_tx_number = int(
                self.prepare_request(
                    "/blockmaxtxnumber/get/",
                    type="get",
                ).text)
            self.max_data_size = int(
                self.prepare_request(
                    "/blockmaxdatasize/get/",
                    type="get",
                ).text)

            if total_check is not None:
                self.total_check = total_check
            else:
                self.total_check = self.prepare_request(
                    "/blockjustonetx/get/",
                    type="get",
                ).text

                if "true" in self.total_check:
                    self.total_check = False
                else:
                    self.total_check = True

            self.original_wait_amoount = copy.copy(self.wait_amount)

            if self.total_check:
                self.check_thread = perpetualTimer(self.original_wait_amoount,
                                                   checker, self)
                self.wait_amount = 0
        except:
            logger.error("Network is not active")
            self.close()
            sys.exit()

        self.host = backup_host
        self.port = backup_port

        logger.info(f"Integration of {self.app_name} is started")

    def init_api(self):
        try:
            self.prepare_request("/", "get")
        except Exception as e:
            self.start_api()

    def start_api(self):
        backup = sys.argv
        sys.argv = [sys.argv[0]]

        self.api = start(host=self.host, port=self.port, test=True)

        self.api_thread = threading.Thread(target=self.api.run)
        self.api_thread.start()
        sys.argv = backup

    def close(self):
        DeleteCommander(self.commander) if not self.commander is None else None
        if self.check_thread is not None:
            while len(self.sended_txs) > 0:
                time.sleep(10)
            self.check_thread.cancel()
        if self.api is not None:
            self.api.close()

    def disable_cache(self):
        self.cache_true = False
        self.cache = []

    def get_cache(self):
        if self.cache_true == False:
            self.cache = []
            return

        os.chdir(get_config()["main_folder"])

        if not os.path.exists(f"db/remote_app_cache/{self.cache_name}.cache"):
            self.cache = []
            self.save_cache()
        with open(f"db/remote_app_cache/{self.cache_name}.cache",
                  "r") as cache:
            self.cache = json.load(cache)

        self.backward_support_cache()
        self.save_cache()

    def backward_support_cache(self):
        for each_cache in self.cache:
            if len(self.cache[self.cache.index(each_cache)]) != 96:
                # remoe the cache
                self.cache.remove(each_cache)

    def save_cache(self):
        if self.cache_true == False:
            self.get_cache()
            return
        self.backward_support_cache()
        os.chdir(get_config()["main_folder"])
        with open(f"db/remote_app_cache/{self.cache_name}.cache",
                  "w") as cache:
            json.dump(self.cache, cache)

    def delete_cache(self):
        os.chdir(get_config()["main_folder"])
        os.remove(f"db/remote_app_cache/{self.cache_name}.cache")

    def prepare_request(self, end_point, type, data=None) -> requests.Response:
        """
        :param end_point: The end point of the request
        :param type: The type of the request (get, post)
        :param data: The data of the request
        :return: The response of the request
        """
        api = f"http://{self.host}:{self.port}"
        response = None
        if type == "post":
            response = requests.post(api + end_point, data=data)
        elif type == "get":
            response = requests.get(api + end_point)

        return response

    def send_forcer(self, action, app_data, to_user, retrysecond):
        stop = False
        while stop == False:
            stop = self.send(action, app_data, to_user, force=False)
            time.sleep(retrysecond)

    def send(self,
             action,
             app_data,
             to_user,
             amount=None,
             force=True,
             retrysecond=10) -> bool:
        """
        :param action: The action of the app
        :param app_data: The data of the app
        :param to_user: The user to send the data to
        """

        if time.time() - self.last_sended < self.wait_amount:
            time.sleep(self.wait_amount - (time.time() - self.last_sended))

        self.host = copy.copy(self.first_host)
        self.port = copy.copy(self.first_port)

        self.init_api()

        data = {"action": self.app_name + action, "app_data": app_data}

        system_length = len(
            json.dumps({
                "action": self.app_name + action,
                "app_data": ""
            }))

        true_length = (self.max_data_size / self.max_tx_number -
                       system_length) - 10

        if len(app_data) > true_length:
            backup_checking = copy.copy(self.checking)
            self.checking = False
            # generate random charactere
            rando = ""
            for i in range(5):
                rando += random.choice(string.ascii_letters)

            split_random = rando + "-"

            self.send(
                action=action,
                app_data=f"split-0-{split_random}",
                to_user=to_user,
                force=force,
                retrysecond=retrysecond,
            )
            len_split_char = len(f"split--{split_random}-")

            total_size_of_an_data = len(
                app_data) + len_split_char + system_length

            how_many_parts = (
                int(math.ceil(
                    (len(app_data) + len_split_char) / true_length)) + 1)

            how_many_parts = int(
                math.ceil(
                    (len(app_data) + len_split_char + len(str(how_many_parts)))
                    / true_length))

            splitted_data = []
            split_length = true_length - len_split_char

            for i in range(how_many_parts):
                # split to part of app_data and app_data is an string
                part = app_data[i * int(split_length):i * int(split_length) +
                                int(split_length)]

                splitted_data.append(part)

            for each_data in splitted_data:
                self.send(
                    action=action,
                    app_data=
                    f"split-{2+splitted_data.index(each_data)}-{split_random}{each_data}",
                    to_user=to_user,
                    force=force,
                    retrysecond=retrysecond,
                )

            self.checking = backup_checking
            checker(self) if self.check_thread is None else None

            self.send(
                action=action,
                app_data=f"split-1-{split_random}",
                to_user=to_user,
                force=force,
                retrysecond=retrysecond,
            )
            return True

        data = json.dumps(data)

        request_body = {
            "password": self.password,
            "to_user": to_user,
            "data": data,
        }

        alread_in_sended = False
        for tx in self.sended_txs:
            if (tx[0] == action and tx[1] == app_data and tx[2] == to_user
                    and tx[3] == amount and tx[4] == force
                    and tx[5] == retrysecond and tx[6] == data):
                alread_in_sended = True
        if not alread_in_sended:
            self.sended_txs.append(
                [action, app_data, to_user, amount, force, retrysecond, data])

        if amount is not None:
            request_body["amount"] = amount

        response = self.prepare_request("/send/",
                                        type="post",
                                        data=request_body)

        if "false" in response.text:
            logger.error("Error on sending message")
            if force:
                logger.info("Trying to send again")
                return self.send_forcer(action, app_data, to_user, retrysecond)
            return False
        else:
            logger.info(
                f"Message sent: app_name:{self.app_name} action:{action} data: {data} to: {to_user}"
            )
            time.sleep(1)
            self.last_sended = time.time()
            if self.checking and self.check_thread is None:
                checker(self)
            return True

    def get_(
        self,
        get_all,
        disable_caches,
        disable_sended_not_validated,
        force_sended,
        raw_data_return=False,
        raw_datas=None,
    ):
        self.get_cache() if not disable_caches else None
        response = self.prepare_request("/transactions/received", type="get")
        transactions = response.json()

        transactions_sended = {}
        transactions_sended_not_validated = {}

        if self.sended or force_sended:
            response = self.prepare_request("/transactions/sended/validated",
                                            type="get")
            transactions_sended = response.json()

        if self.sended_not_validated and not disable_sended_not_validated:
            response = self.prepare_request(
                "/transactions/sended/not_validated", type="get")
            transactions_sended_not_validated = response.json()

        if raw_data_return:
            return transactions, transactions_sended, transactions_sended_not_validated

        if raw_datas is not None:
            for data in raw_datas[0]:
                with contextlib.suppress(TypeError):
                    transactions[data] = raw_datas[0][data]
            for data in raw_datas[1]:
                with contextlib.suppress(TypeError):
                    transactions_sended[data] = raw_datas[1][data]
            for data in raw_datas[2]:
                with contextlib.suppress(TypeError):
                    transactions_sended_not_validated[data] = raw_datas[2][
                        data]

        new_dict = {}
        commanders = GetCommander()
        for transaction in transactions:
            if (transactions[transaction]["transaction"]["signature"]
                    in self.cache) and not get_all:
                continue
            else:
                if (transactions[transaction]["transaction"]["toUser"]
                        == wallet_import(-1, 3)
                        or Address(transactions[transaction]["transaction"]
                                   ["fromUser"]) in commanders):
                    new_dict[transaction] = transactions[transaction]
                    the_tx = Transaction.load_json(
                        transactions[transaction]["transaction"])
                    if not transactions[transaction]["transaction"][
                            "data"] == "NP":
                        with contextlib.suppress(json.decoder.JSONDecodeError):
                            transactions[transaction]["transaction"][
                                "data"] = json.loads(transactions[transaction]
                                                     ["transaction"]["data"])
                        if not transactions[transaction]["transaction"][
                                "data"]["app_data"].startswith("split-"):
                            self.cache.append(
                                transactions[transaction]["transaction"]
                                ["signature"]) if not disable_caches else None

                            SavetoMyTransaction(
                                the_tx) if not get_all else None
                            ValidateTransaction(
                                the_tx) if not get_all else None
                    else:
                        SavetoMyTransaction(the_tx) if not get_all else None
                        ValidateTransaction(the_tx) if not get_all else None
                        self.cache.append(
                            transactions[transaction]["transaction"]
                            ["signature"]) if not disable_caches else None
                elif transactions[transaction]["transaction"][
                        "fromUser"] == wallet_import(-1, 0):
                    transactions_sended[transaction] = transactions[
                        transaction]

        for transaction in transactions_sended:
            if self.sended or force_sended:
                if (transactions_sended[transaction]["transaction"]
                    ["signature"] in self.cache) and not get_all:
                    continue
                else:
                    if transactions_sended[transaction]["transaction"][
                            "fromUser"] == wallet_import(-1, 0):
                        new_dict[transaction] = transactions_sended[
                            transaction]
                        the_tx = Transaction.load_json(
                            transactions_sended[transaction]["transaction"])

                        if (not transactions_sended[transaction]["transaction"]
                            ["data"] == "NP"):
                            with contextlib.suppress(
                                    json.decoder.JSONDecodeError):
                                transactions_sended[transaction][
                                    "transaction"]["data"] = json.loads(
                                        transactions_sended[transaction]
                                        ["transaction"]["data"])
                            if not transactions_sended[transaction][
                                    "transaction"]["data"][
                                        "app_data"].startswith("split-"):
                                self.cache.append(
                                    transactions_sended[transaction]
                                    ["transaction"]["signature"]
                                ) if not disable_caches else None

                                SavetoMyTransaction(
                                    the_tx) if not get_all else None
                                ValidateTransaction(
                                    the_tx) if not get_all else None
                        else:
                            SavetoMyTransaction(
                                the_tx) if not get_all else None
                            ValidateTransaction(
                                the_tx) if not get_all else None
                            self.cache.append(
                                transactions_sended[transaction]["transaction"]
                                ["signature"]) if not disable_caches else None
        split_not_validated = []
        for transaction in transactions_sended_not_validated:
            if self.sended_not_validated and not disable_sended_not_validated:
                if (transactions_sended_not_validated[transaction]
                    ["transaction"]["signature"]
                        in self.cache) and not get_all:
                    continue
                else:
                    if transactions_sended_not_validated[transaction][
                            "transaction"]["fromUser"] == wallet_import(-1, 0):
                        the_tx = Transaction.load_json(
                            transactions_sended_not_validated[transaction]
                            ["transaction"])

                        new_dict[
                            transaction] = transactions_sended_not_validated[
                                transaction]
                        if (not transactions_sended_not_validated[transaction]
                            ["transaction"]["data"] == "NP"):
                            with contextlib.suppress(
                                    json.decoder.JSONDecodeError):
                                transactions_sended_not_validated[transaction][
                                    "transaction"]["data"] = json.loads(
                                        transactions_sended_not_validated[
                                            transaction]["transaction"]
                                        ["data"])
                            if not transactions_sended_not_validated[
                                    transaction]["transaction"]["data"][
                                        "app_data"].startswith("split-"):
                                self.cache.append(
                                    transactions_sended_not_validated[
                                        transaction]["transaction"]
                                    ["signature"]
                                ) if not disable_caches else None

                                split_not_validated.append(
                                    transactions_sended_not_validated[
                                        transaction]["transaction"]
                                    ["signature"])
                                SavetoMyTransaction(
                                    the_tx) if not get_all else None
                        else:
                            SavetoMyTransaction(
                                the_tx) if not get_all else None
                            self.cache.append(
                                transactions_sended_not_validated[transaction]
                                ["transaction"]
                                ["signature"]) if not disable_caches else None

        self.save_cache() if not disable_caches else None

        last_list = []

        for transaction in new_dict:
            with contextlib.suppress(TypeError):
                if not new_dict[transaction]["transaction"]["data"] == "NP":
                    if (self.app_name in new_dict[transaction]["transaction"]
                        ["data"]["action"]):
                        last_list.append(new_dict[transaction]["transaction"])

        splits = []
        # finding transactions that data start with split
        new_a_last_list = copy.copy(last_list)
        for transaction in last_list:
            # check new_dict[transaction]["transaction"]["data"] is start with split

            if transaction["data"]["app_data"].startswith("split-0"):
                the_split = splitted_data(
                    transaction["data"]["app_data"].split("-")[2])
                the_split.data_original.append(transaction)
                splits.append(the_split)
                new_a_last_list.remove(transaction) if not get_all else None

        last_list = new_a_last_list
        new_last_list = copy.copy(last_list)

        for split in splits:
            for transaction in last_list:
                if transaction["data"]["app_data"].startswith("split-"):
                    if not transaction["data"]["app_data"].startswith(
                            "split-0") and not transaction["data"][
                                "app_data"].startswith("split-1"):
                        if transaction["data"]["app_data"].split(
                                "-")[2] == split.split:
                            split.data.append(transaction["data"]["app_data"])
                            split.data_original.append(transaction)
                            with contextlib.suppress(ValueError):
                                new_last_list.remove(
                                    transaction) if not get_all else None

        last_list = new_last_list

        new_last_list_2 = copy.copy(last_list)
        for transaction in last_list:
            finded = False

            for split in splits:
                if transaction["data"]["app_data"].startswith("split-1"):
                    if transaction["data"]["app_data"].split(
                            "-")[2] == split.split:
                        finded = True
                        split.validated = True

                        split.main_data = copy.copy(transaction)
                        split.data_original.append(copy.copy(transaction))
                        new_last_list_2.remove(
                            transaction) if not get_all else None

                        break
        last_list = new_last_list_2

        new_splits = copy.copy(splits)
        new_last_list_3 = copy.copy(last_list)
        for split in splits:
            if not split.validated:
                for each_data in split.data:
                    for transaction in last_list:
                        if each_data == transaction["data"]["app_data"]:
                            new_last_list_3.remove(
                                transaction) if not get_all else None

                            break

                new_splits.remove(split)

        last_list = new_last_list_3

        splits = new_splits

        for split in splits:
            if split.validated:
                for each_original in split.data_original:
                    self.cache.append(each_original["signature"]
                                      ) if not disable_caches else None
                    SavetoMyTransaction(the_tx)
                    if not each_original["signature"] in split_not_validated:
                        ValidateTransaction(the_tx) if not get_all else None
                    if Address(each_original["fromUser"]) == wallet_import(
                            -1, 3):
                        SendedTransaction(Transaction.load_json(
                            each_original)) if not get_all else None
                for each_data in split.data:
                    split.main_data["data"]["app_data"] += each_data
                    split.main_data["data"]["app_data"] = split.main_data[
                        "data"]["app_data"].replace(f"split-1-{split.split}-",
                                                    "")
                    for i in range(len(split.data)):
                        split.main_data["data"]["app_data"] = split.main_data[
                            "data"]["app_data"].replace(
                                f"split-{i+2}-{split.split}-", "")
                last_list.append(split.main_data)

        self.save_cache() if not disable_caches else None

        result = []

        for transaction in last_list:
            transaction["fromUser"] = Address(transaction["fromUser"])
            if transaction["fromUser"] == wallet_import(-1, 3):
                the_tx = Transaction.load_json(transaction)
                if the_settings()["baklava"] and not transaction["data"][
                        "app_data"].startswith("split-"):
                    SendedTransaction(the_tx) if not get_all else None
                result.append(transaction)

            elif (transaction["toUser"] == wallet_import(-1, 3)
                  or transaction["fromUser"] in commanders):
                result.append(transaction)

        for transaction in result[:]:
            if transaction["data"]["app_data"].startswith("split-"):
                result.remove(transaction) if not get_all else None

        if not len(result) == 0:
            logger.info("New datas received")

        return result

    def get(
        self,
        get_all=False,
        disable_caches=False,
        from_thread=False,
        disable_sended_not_validated=False,
        force_sended=False,
    ):
        self.host = copy.copy(self.first_host)
        self.port = copy.copy(self.first_port)

        backup_host = copy.copy(self.host)
        backup_port = copy.copy(self.port)

        if ((self.sended or force_sended) and self.check_thread is not None
                and not from_thread):
            while len(self.sended_txs) > 0:
                time.sleep(10)

        if the_settings()["baklava"]:
            self.host = "test_net.1.naruno.org"
            self.port = 8000

        baklava_datas = None

        with contextlib.suppress(Exception):
            baklava_datas = self.get_(
                get_all=get_all,
                disable_caches=disable_caches,
                disable_sended_not_validated=disable_sended_not_validated,
                force_sended=force_sended,
                raw_data_return=True,
            )
        self.host = backup_host
        self.port = backup_port

        self.init_api()

        second = self.get_(
            get_all=get_all,
            disable_caches=disable_caches,
            disable_sended_not_validated=disable_sended_not_validated,
            force_sended=force_sended,
            raw_datas=baklava_datas,
        )

        the_list = second

        with contextlib.suppress(TypeError):
            if "print" in inspect.stack()[1].code_context[0]:
                total = ""
                for data in the_list:
                    fromUser = data["fromUser"]
                    toUser = data["toUser"]
                    action = data["data"]["action"].replace(self.app_name, "")
                    data = data["data"]["app_data"]
                    total += f"\n-----\nFrom: {fromUser}, To: {toUser} \nApp Name: {self.app_name}, Action: {action} \nData: {data}\n-----"
                return total
        return the_list



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import contextlib
import json
import time

from naruno.accounts.commanders.delete_commander import DeleteCommander
from naruno.lib.log import get_logger

logger = get_logger("REMOTE_APP")


def checker(integration, logger=logger):
    """
    Retrieves new transactions, checks if they match previously sent
    transactions, and resends any transactions that have not been
    validated. Uses a combination of loops and try/except blocks to
    iterate over transactions and catch any errors that may occur.

    Args:
        integration: The Integration object instance

    Returns:
        None
    """

    try:
        logger.debug("Starting checker function")

        # Sleep for a certain amount of time
        time.sleep(integration.wait_amount)

        # Get new transactions
        new_txs = integration.get(
            get_all=True,
            disable_caches=True,
            from_thread=True,
            disable_sended_not_validated=True,
            force_sended=True,
        )

        # Log how many new transactions were retrieved
        logger.debug(f"Retrieved {len(new_txs)} new transactions")

        # Loop through previously sent transactions and check if they have been validated
        for sended_tx in integration.sended_txs[:integration.max_tx_number //
                                                2]:
            in_get = False
            with contextlib.suppress(ValueError):
                # If the transaction has been validated, remove it from the list of sent transactions
                integration.sended_txs.remove(sended_tx)

            # Check if the validated transaction matches the sent transaction
            for vaidated_tx in new_txs:
                if (vaidated_tx["toUser"] == sended_tx[2]
                        and vaidated_tx["data"]["action"] == json.loads(
                            sended_tx[6])["action"]
                        and vaidated_tx["data"]["app_data"] == json.loads(
                            sended_tx[6])["app_data"]):
                    in_get = True
            if not in_get:
                # If the validated transaction does not match the sent transaction, resend the transaction
                logger.warning(
                    f"Transaction not found, resending: {sended_tx}")

                integration.send(
                    sended_tx[0],
                    sended_tx[1],
                    sended_tx[2],
                    sended_tx[3],
                    sended_tx[4],
                    sended_tx[5],
                )

        logger.debug("Exiting checker function")

    except Exception as e:
        # Catch any errors that occur during the function and log them
        logger.error(f"An error occured in the checker function: {e}")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from naruno.lib.log import get_logger
from naruno.lib.settings_system import change_wallet
from naruno.wallet.get_saved_wallet import get_saved_wallet
from naruno.wallet.print_wallets import print_wallets

logger = get_logger("WALLET")


def wallet_selector(new_wallet_number):
    """
    Changes the current wallet.
    """

    all_wallets = list(get_saved_wallet())
    if not all_wallets:
        logger.error("There is no wallet")
        return None

    new_wallet_from_function = None
    try:
        if int(new_wallet_number) in list(range(len(all_wallets))):
            new_wallet_from_function = change_wallet(int(new_wallet_number))
            logger.info("New Wallets:")
            print_wallets()

        else:
            logger.error("There is no such wallet")
            new_wallet_from_function = None

    except:
        logger.error("This is not a number")
        new_wallet_from_function = False
    return new_wallet_from_function



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from naruno.blockchain.block.block_main import Block
from naruno.blockchain.candidate_block.candidate_block_main import \
    candidate_block
from naruno.lib.log import get_logger

logger = get_logger("CONSENSUS_SECOND_ROUND")


def process_candidate_blocks_hashes(block: Block,
                                    candidate_class: candidate_block,
                                    unl_nodes: dict) -> dict:
    logger.info("Control process of candidate block hashes is started.")

    current_hash = {"signature": "A", "hash": False, "previous_hash": False}
    previous_hash = {"signature": "A", "hash": False, "previous_hash": False}

    for candidate_block_hash in candidate_class.candidate_block_hashes[:]:
        logger.debug(f"Candidate block hash {candidate_block_hash}")

        tx_valid = 1

        for other_block in candidate_class.candidate_block_hashes[:]:
            if (candidate_block_hash != other_block
                    and candidate_block_hash["hash"] == other_block["hash"]):
                tx_valid += 1

        logger.debug(f"Hash valid of  {candidate_block_hash} : {tx_valid}")

        if tx_valid > (((len(unl_nodes)+1) * 50) / 100):
            logger.info(
                f"candidate_block_hash: {candidate_block_hash} is validated.")
            current_hash = candidate_block_hash

    for candidate_block_hash in candidate_class.candidate_block_hashes[:]:
        logger.debug(f"Candidate block hash previous_hash {candidate_block_hash}")

        tx_valid = 1

        for other_block in candidate_class.candidate_block_hashes[:]:
            if (candidate_block_hash != other_block
                    and candidate_block_hash["previous_hash"] == other_block["previous_hash"]):
                tx_valid += 1
        logger.debug(f"Hash valid of previous_hash  {candidate_block_hash} : {tx_valid}")
        if tx_valid > (((len(unl_nodes)+1) * 50) / 100):
            logger.info(
                f"candidate_block_hash previous_hash: {candidate_block_hash} is validated.")
            previous_hash = candidate_block_hash



    if current_hash != {"signature": "A", "hash": False, "previous_hash": False} or previous_hash != {"signature": "A", "hash": False, "previous_hash": False}:
        if current_hash["signature"] == "self":
            for other in candidate_class.candidate_block_hashes[:]:
                if current_hash != other and current_hash["hash"] == other["hash"]:
                    current_hash = other
                    break
                    
        if previous_hash["signature"] == "self":
            for other in candidate_class.candidate_block_hashes[:]:
                if previous_hash != other and previous_hash["previous_hash"] == other["previous_hash"]:
                    previous_hash = other
                    break        
        return {"hash": current_hash, "previous_hash": previous_hash}

    logger.debug("All candidate_block_hashes can not be validated.")
    return {"hash": {"hash": False}, "previous_hash": {"previous_hash": False}}



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from naruno.config import *
from naruno.lib.encryption import encrypt
from naruno.wallet.ellipticcurve.privateKey import PrivateKey
from naruno.wallet.get_saved_wallet import get_saved_wallet
from naruno.wallet.save_wallet_list import save_to_wallet_list


def wallet_create(password, save=True):

    my_private_key = PrivateKey()
    my_public_key = my_private_key.publicKey()

    if save != True:
        return my_private_key
    encrypted_key = (encrypt(my_private_key.toPem(), password)
                     if list(get_saved_wallet()) else my_private_key.toPem())

    del my_private_key
    save_to_wallet_list(my_public_key.toPem(), encrypted_key, password)
    return encrypted_key



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from naruno.lib.mix.merkle_root import MerkleTree


def TransactionsHash(block):
    """
    Calculates and returns the hash of the validating list.
    """
    tx_list = [element.signature for element in block.validating_list]
    return MerkleTree(tx_list).getRootHash() if tx_list else "0"



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import contextlib
import threading

from naruno.blockchain.block.block_main import Block
from naruno.blockchain.block.get_block import GetBlock
from naruno.blockchain.block.save_block import SaveBlock
from naruno.blockchain.candidate_block.candidate_block_main import \
    candidate_block
from naruno.consensus.finished.finished_main import finished_main
from naruno.consensus.ongoing.ongoing_main import ongoing_main
from naruno.lib.log import get_logger
from naruno.lib.perpetualtimer import perpetualTimer
from naruno.node.client.client import client
from naruno.node.server.server import server
from naruno.transactions.cleaner import Cleaner
from naruno.transactions.pending.get_pending import GetPending

logger = get_logger("CONSENSUS")



def send_block_hash(
    block: Block,
    the_server: server = None,
    send_block_hash_error: bool = False,

    ):

    logger.debug("Our block hash is sending to the unl nodes")
    try:
        the_server.send_my_block_hash(block)
        if send_block_hash_error:
            raise Exception("Block hash sending error")
    except Exception as e:
        logger.error(f"Block hash sending error: {e}")


#!/usr/bin/python3
# -*- coding: utf-8 -*-
from re import search


def getPemContent(pem, template):
    pattern = template.format(content="(.*)")
    return search("".join(pattern.splitlines()),
                  "".join(pem.splitlines())).group(1)


def createPem(content, template):
    lines = [content[start:start + 64] for start in range(0, len(content), 64)]
    return template.format(content="\n".join(lines))



#!/usr/bin/python3
# -*- coding: utf-8 -*-
from naruno.wallet.ellipticcurve.utils.binary import base64FromByteString
from naruno.wallet.ellipticcurve.utils.binary import byteStringFromBase64
from naruno.wallet.ellipticcurve.utils.binary import byteStringFromHex
from naruno.wallet.ellipticcurve.utils.binary import hexFromByteString
from naruno.wallet.ellipticcurve.utils.compatibility import *
from naruno.wallet.ellipticcurve.utils.der import DerFieldType
from naruno.wallet.ellipticcurve.utils.der import encodeConstructed
from naruno.wallet.ellipticcurve.utils.der import encodePrimitive
from naruno.wallet.ellipticcurve.utils.der import parse


class Signature:

    def __init__(self, r, s, recoveryId=None):
        self.r = r
        self.s = s
        self.recoveryId = recoveryId

    def toDer(self, withRecoveryId=False):
        hexadecimal = self._toString()
        encodedSequence = byteStringFromHex(hexadecimal)
        if not withRecoveryId:
            return encodedSequence

    def toBase64(self, withRecoveryId=False):
        return base64FromByteString(self.toDer(withRecoveryId))

    @classmethod
    def fromDer(cls, string, recoveryByte=False):
        recoveryId = None

        hexadecimal = hexFromByteString(string)
        return cls._fromString(string=hexadecimal, recoveryId=recoveryId)

    @classmethod
    def fromBase64(cls, string, recoveryByte=False):
        der = byteStringFromBase64(string)
        return cls.fromDer(der, recoveryByte)

    def _toString(self):
        return encodeConstructed(
            encodePrimitive(DerFieldType.integer, self.r),
            encodePrimitive(DerFieldType.integer, self.s),
        )

    @classmethod
    def _fromString(cls, string, recoveryId=None):
        r, s = parse(string)[0]
        return Signature(r=r, s=s, recoveryId=recoveryId)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import time

from naruno.blockchain.block.block_main import Block
from naruno.lib.log import get_logger
from naruno.blockchain.block.save_block import SaveBlock
from naruno.transactions.pending_to_validating import PendingtoValidating

logger = get_logger("CONSENSUS_FIRST_ROUND")


def time_difference_check(
    block: Block, 
    return_result=False,
    custom_TEMP_ACCOUNTS_PATH: str = None,
    custom_TEMP_BLOCK_PATH: str = None,
    custom_TEMP_BLOCKSHASH_PATH: str = None,
    custom_TEMP_BLOCKSHASH_PART_PATH: str = None,    
    ) -> bool:

    the_time = block.start_time + block.round_1_time
    current_time = int(time.time())
    logger.info("Round 1 time control started")
    logger.debug(f"current_time: {current_time}")
    logger.debug(f"the_time: {the_time}")
    if current_time >= the_time:
        logger.info("Time is true")
        return True
    else:
        if not current_time >= the_time - (block.round_1_time / 2):
            PendingtoValidating(block)
            SaveBlock(
                block,
                custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH,
                custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
                custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH,
                custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH,
            )     
        logger.info("Time is not true")
        return False if return_result is False else the_time



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import time

from naruno.blockchain.block.block_main import Block
from naruno.consensus.rounds.round_1.process.transactions.checks.duplicated import Remove_Duplicates
from naruno.lib.log import get_logger
from naruno.transactions.transaction import Transaction

logger = get_logger("BLOCKCHAIN")


def shares(block: Block, custom_shares=None, custom_fee_address=None, dont_clean=False) -> list:
    """
    It returns the transactions that needed for locked shares distribution.
    """
    logger.info("Share distribution started.")
    logger.debug(f"block.validating_list: {block.validating_list}")
    the_shares = block.shares if custom_shares is None else custom_shares
    the_fee_address = (block.fee_address
                       if custom_fee_address is None else custom_fee_address)
    logger.debug(f"block.sequence_number: {block.sequence_number}")
    logger.debug(f"the_shares: {the_shares}")
    logger.debug(f"the_fee_address: {the_fee_address}")

    tx_list = []

    the_time = block.genesis_time
    logger.debug(f"the_time: {the_time}")

    for share in the_shares:
        rate = block.sequence_number / share[2]
        if rate.is_integer() and rate != 0.0:
            if not block.sequence_number > share[3]:
                tx_list.append(
                    Transaction(
                        0,
                        "NARUNO",
                        "NARUNOB",
                        share[0],
                        "NP",
                        share[1],
                        0,
                        the_time,
                    ))

    fee = 0
    if not dont_clean:
        block = Remove_Duplicates(block)
    block.validating_list = sorted(block.validating_list,
                                   key=lambda x: x.fromUser)    
    for tx in block.validating_list:
        if not "NARUNO" in tx.signature:
            fee += tx.transaction_fee
    if fee > 0:
        tx_list.append(
            Transaction(
                0,
                "NARUNO",
                "NARUNOA",
                the_fee_address,
                "NP",
                fee,
                0,
                the_time,
            ))

    


    return tx_list



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# Elliptic Curve Equation
#
# y^2 = x^3 + A*x + B (mod P)
#
from naruno.wallet.ellipticcurve.point import Point


class CurveFp:

    def __init__(self, A, B, P, N, Gx, Gy, name, oid, nistName=None):
        self.A = A
        self.B = B
        self.P = P
        self.N = N
        self.G = Point(Gx, Gy)
        self.name = name
        self.nistName = nistName
        self.oid = oid  # ASN.1 Object Identifier

    def length(self):
        return (1 + len("%x" % self.N)) // 2


secp256k1 = CurveFp(
    name="secp256k1",
    A=0x0000000000000000000000000000000000000000000000000000000000000000,
    B=0x0000000000000000000000000000000000000000000000000000000000000007,
    P=0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFFFFFC2F,
    N=0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEBAAEDCE6AF48A03BBFD25E8CD0364141,
    Gx=0x79BE667EF9DCBBAC55A06295CE870B07029BFCDB2DCE28D959F2815B16F81798,
    Gy=0x483ADA7726A3C4655DA4FBFC0E1108A8FD17B448A68554199C47D08FFB10D4B8,
    oid=[1, 3, 132, 0, 10],
)

prime256v1 = CurveFp(
    name="prime256v1",
    nistName="P-256",
    A=0xFFFFFFFF000000010000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFC,
    B=0x5AC635D8AA3A93E7B3EBBD55769886BC651D06B0CC53B0F63BCE3C3E27D2604B,
    P=0xFFFFFFFF000000010000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFF,
    N=0xFFFFFFFF00000000FFFFFFFFFFFFFFFFBCE6FAADA7179E84F3B9CAC2FC632551,
    Gx=0x6B17D1F2E12C4247F8BCE6E563A440F277037D812DEB33A0F4A13945D898C296,
    Gy=0x4FE342E2FE1A7F9B8EE7EB4A7C0F9E162BCE33576B315ECECBB6406837BF51F5,
    oid=[1, 2, 840, 10045, 3, 1, 7],
)

p256 = prime256v1

supportedCurves = [
    secp256k1,
    prime256v1,
]

_curvesByOid = {tuple(curve.oid): curve for curve in supportedCurves}


def getCurveByOid(oid):

    return _curvesByOid[oid]



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import time

from naruno.accounts.get_balance import GetBalance
from naruno.accounts.get_sequence_number import GetSequanceNumber
from naruno.lib.log import get_logger
from naruno.transactions.pending.get_pending import GetPending

logger = get_logger("TRANSACTIONS")


def Check_Datas(
    block,
    transaction,
    custom_current_time=None,
    custom_balance=None,
    custom_sequence_number=None,
    custom_PENDING_TRANSACTIONS_PATH=None,
    custom_account_list=None,
    disable_already_in=False,
):
    """
    Check if the transaction datas are valid
    """

    if not disable_already_in:
        pending_transactions = GetPending(
            custom_PENDING_TRANSACTIONS_PATH=custom_PENDING_TRANSACTIONS_PATH)
        for already_tx in pending_transactions + block.validating_list:
            if already_tx.signature == transaction.signature:
                logger.error("Transaction is already in the pending list")
                return False

        for tx in pending_transactions + block.validating_list:
            if (tx.fromUser == transaction.fromUser) and block.just_one_tx:
                if tx.signature != transaction.signature:
                    if transaction.transaction_time < tx.transaction_time:
                        pass
                    elif transaction.transaction_time == tx.transaction_time:
                        if transaction.sequence_number < tx.sequence_number:
                            pass
                        else:
                            logger.info("Multiple transaction in one account")
                            return False
                    else:
                        logger.info("Multiple transaction in one account")
                        return False
    if not disable_already_in:
        balance = (GetBalance(
            transaction.fromUser,
            block=block,
        ) if custom_balance is None else custom_balance)
        if balance >= (float(transaction.amount) +
                    float(transaction.transaction_fee)):
            pass
        else:
            logger.error("Balance is not valid")
            return False

        if transaction.amount >= block.minumum_transfer_amount:
            pass
        else:
            if (GetBalance(
                    transaction.toUser,
                    account_list=custom_account_list,
                    dont_convert=True,
                    block=block,
            ) >= 0):
                pass
            else:
                logger.error("Minimum transfer amount is not reached")
                return False

    if transaction.transaction_fee >= block.transaction_fee:
        pass
    else:
        logger.error(
            f"Transaction fee is not reached {transaction.transaction_fee}-{block.transaction_fee}"
        )
        return False

    if not disable_already_in:
        get_sequence_number = (GetSequanceNumber(transaction.fromUser, block=block)
                            if custom_sequence_number is None else
                            custom_sequence_number)
        if transaction.sequence_number == (get_sequence_number + 1):
            pass
        else:
            logger.error("Sequance number is not valid")
            return False

    current_time = (int(time.time())
                    if custom_current_time is None else custom_current_time)
    if (current_time -
            transaction.transaction_time) <= block.transaction_delay_time:
        pass
    else:
        logger.error("Transaction time is not valid")
        return False

    return True



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import os
import sys

sys.path.append(os.path.join(os.path.dirname(__file__), "..", "..", ".."))

from naruno.lib.performance_analyzers.accounts import \
    Accounts_IO_Performance_Analyzer
from naruno.lib.performance_analyzers.block import \
    Block_IO_Performance_Analyzer
from naruno.lib.performance_analyzers.blockshash import \
    Blockshash_IO_Performance_Analyzer
from naruno.lib.performance_analyzers.blockshash_part import \
    Blockshash_part_IO_Performance_Analyzer
from naruno.lib.performance_analyzers.transactions import \
    Transactions_IO_Performance_Analyzer


def heartbeat_generic_db_analyzer():
    the_block = Block_IO_Performance_Analyzer()
    the_accounts = Accounts_IO_Performance_Analyzer()
    the_blockshash = Blockshash_IO_Performance_Analyzer()
    the_blockshash_part = Blockshash_part_IO_Performance_Analyzer()
    the_transactions = Transactions_IO_Performance_Analyzer()

    the_block_analysis = the_block.analyze()
    the_accounts_analysis = the_accounts.analyze()
    the_blockshash_analysis = the_blockshash.analyze()
    the_blockshash_part_analysis = the_blockshash_part.analyze()
    the_transactions_analysis = the_transactions.analyze()

    save_total = (the_block_analysis[0] + the_accounts_analysis[0] +
                  the_blockshash_analysis[0] +
                  the_blockshash_part_analysis[0] +
                  the_transactions_analysis[0])
    get_total = (the_block_analysis[1] + the_accounts_analysis[1] +
                 the_blockshash_analysis[1] + the_blockshash_part_analysis[1] +
                 the_transactions_analysis[1])
    size_total = (the_block_analysis[2] + the_accounts_analysis[2] +
                  the_blockshash_analysis[2] +
                  the_blockshash_part_analysis[2] +
                  the_transactions_analysis[2])

    return [
        [
            save_total,
            get_total,
            size_total,
        ],
        [
            save_total - the_blockshash_part_analysis[0],
            get_total - the_blockshash_part_analysis[1],
            size_total - the_blockshash_part_analysis[2],
        ],
    ]


if __name__ == "__main__":
    print(heartbeat_generic_db_analyzer())



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from io import open

from setuptools import setup

setup(
    author="Naruno Developers",
    author_email="onur.atakan.ulusoy@naruno.org",
    packages=["naruno"],
    name="naruno",
    version="0.57.1",
    url="https://github.com/Naruno/Naruno",
    description=
    "Naruno is a lightning-fast, secure, and scalable blockchain that is able to create transaction proofs and verification via raw data and timestamp. We remove the archive nodes and lazy web3 integrations. With Naruno everyone can get the proof (5-10MB) of their transactions via their nodes and after everyone can use in another node for verification the raw data and timestamp. Also you can integrate your web3 applications with 4 code lines (just python for now) via our remote app system.",
    keywords=[
        "python",
        "cryptography",
        "blockchain",
        "p2p",
        "python3",
        "cryptocurrency",
        "kivy",
        "coin",
        "copilot",
        "fba",
        "dapps",
        "p2p-network",
        "kivymd",
        "blokzinciri",
        "naruno",
        "githubcopilot",
        "blokzincir",
    ],
    long_description_content_type="text/markdown",
    long_description="".join(open("README.md", encoding="utf-8").readlines()),
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "narunogui = naruno.gui.main:start",
            "narunocli = naruno.cli.main:start",
            "narunoapi = naruno.api.main:start",
        ],
    },
    license="MPL-2.0",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)",
        "Programming Language :: Python :: 3",
    ],
    python_requires=">=3.8",
)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import contextlib
import json
from threading import Event, Thread, Timer


class perpetualTimer(Timer):
    """
    It trig the functions at time intervals, independent of the main process.

    perpetualTimer class consists of 3 elements:
      * t: The time interval of perpetualTimer.
      * hFunction: The function to be triggered.
    """

    def __init__(self, interval, function, args=None, kwargs=None):
        Thread.__init__(self)
        self.interval = interval
        self.function = function
        self.args = args if args is not None else []
        self.kwargs = kwargs if kwargs is not None else {}
        self.finished = Event()

        if self.interval != 0:
            self.start()

    def run(self):
        while not self.finished.wait(self.interval):
            with contextlib.suppress(json.decoder.JSONDecodeError):
                self.function(*self.args, **self.kwargs)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import hashlib
import os
import sys
import time

from speed_calculator import calculate

sys.path.append(os.path.join(os.path.dirname(__file__), "..", "..", ".."))

from naruno.accounts.account import Account
from naruno.accounts.get_accounts import GetAccounts
from naruno.accounts.save_accounts import SaveAccounts
from naruno.blockchain.block.block_main import Block
from naruno.blockchain.block.blocks_hash import (GetBlockshash,
                                                 GetBlockshash_part,
                                                 SaveBlockshash,
                                                 SaveBlockshash_part)
from naruno.blockchain.block.get_block import GetBlock
from naruno.blockchain.block.save_block import SaveBlock
from naruno.lib.mix.merkle_root import MerkleTree
from naruno.transactions.transaction import Transaction


class Block_IO_Performance_Analyzer:
    """
    This class is used to analyze the performance of GetBlock
    """

    def __init__(self):
        self.block = Block("test")
        self.block.first_time = False

        self.block.hash = hashlib.sha256("test".encode()).hexdigest()
        self.block.round_2_starting_time = time.time()
        self.block.validated_time = time.time()

        the_transaction_json = {
            "sequence_number": 1,
            "signature":
            "MEUCIHABt7ypkpvFlpqL4SuogwVuzMu2gGynVkrSw6ohZ/GyAiEAg2O3iOei1Ft/vQRpboX7Sm1OOey8a3a67wPJaH/FmVE=",
            "fromUser":
            "MFYwEAYHKoZIzj0CAQYFK4EEAAoDQgAE0AYA7B+neqfUA17wKh3OxC67K8UlIskMm9T2qAR+pl+kKX1SleqqvLPM5bGykZ8tqq4RGtAcGtrtvEBrB9DTPg==",
            "toUser": "onur",
            "data": "blockchain-lab",
            "amount": 5000.0,
            "transaction_fee": 0.02,
            "transaction_time": 1656764224,
        }
        number_of_data = self.block.max_data_size // self.block.max_tx_number
        the_transaction_json["data"] = "a" * number_of_data
        the_transaction = Transaction.load_json(the_transaction_json)
        self.block.validating_list = [
            the_transaction for i in range(self.block.max_tx_number)
        ]

    def analyze(self) -> float:
        """
        This function is used to analyze the performance of GetBlock
        """

        result = (
            calculate(self.save_operation)[0],
            calculate(self.get_operation)[0],
            os.path.getsize("db/Block_Performance_Analyzer_block.pf") /
            1000000,
        )

        os.remove("db/Block_Performance_Analyzer_block.pf")

        return result

    def save_operation(self):
        """
        This function is used to analyze the performance of GetBlock
        """
        SaveBlock(
            self.block,
            custom_TEMP_BLOCK_PATH="db/Block_Performance_Analyzer_block.pf",
        )

        main_list = [
            self.block.previous_hash,
            str(self.block.sequence_number),
            str(self.block.empty_block_number),
            str(self.block.gap_block_number),
            str(self.block.default_transaction_fee),
            str(self.block.default_increase_of_fee),
            str(self.block.default_optimum_transaction_number),
        ]

        MerkleTree(main_list).getRootHash()

    def get_operation(self):
        """
        This function is used to analyze the performance of GetBlock
        """

        GetBlock("db/Block_Performance_Analyzer_block.pf")


if __name__ == "__main__":
    print(Block_IO_Performance_Analyzer().analyze())



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from naruno.blockchain.block.block_main import Block
from naruno.blockchain.candidate_block.candidate_block_main import \
    candidate_block
from naruno.consensus.rounds.round_1.checks.checks_main import \
    round_check
from naruno.consensus.rounds.round_2.checks.checks_main import \
    round_check
from naruno.consensus.rounds.round_2.process.process_main import \
    round_process
from naruno.lib.log import get_logger
from naruno.node.client.client import client
from naruno.node.get_candidate_blocks import GetCandidateBlocks
from naruno.node.server.server import server
from naruno.node.unl import Unl

logger = get_logger("CONSENSUS_SECOND_ROUND")


def consensus_round_2(
    block: Block,
    candidate_class: candidate_block = None,
    unl_nodes: dict = None,
    custom_server: server = None,
    custom_unl: client = None,
    custom_TEMP_BLOCK_PATH: str = None,
    custom_TEMP_ACCOUNTS_PATH: str = None,
    custom_TEMP_BLOCKSHASH_PATH: str = None,
    custom_TEMP_BLOCKSHASH_PART_PATH: str = None,
) -> bool:
    """
    At this stage of the consensus process,
    The blocks (block hashes) of our and the
    unl nodes are compared and the block accepted
    by majority approving.

    Inputs:
      * block: The block (class) we want consensus
      round 2 to be done
    """

    logger.info(
        f"BLOCK#{block.sequence_number}:{block.empty_block_number} second round is starting"
    )

    unl_nodes = Unl.get_unl_nodes() if unl_nodes is None else unl_nodes
    candidate_class = (GetCandidateBlocks(block=block)
                       if candidate_class is None else candidate_class)
    logger.debug(f"unl_nodes: {unl_nodes}")
    logger.debug(f"candidate_class: {candidate_class.__dict__}")

    if round_check(block, candidate_class, unl_nodes):
        round_process(
            block,
            candidate_class,
            unl_nodes,
            custom_server=custom_server,
            custom_unl=custom_unl,
            custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH,
            custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
            custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH,
            custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH,
        )
        
        logger.info("Round 2 check is True")
        return True
    else:
        logger.warning("Round 2 check is False")
        return False




import copy
import os
import shutil
import sys
from typing import Union

sys.path.append(os.path.join(os.path.dirname(__file__), "..", "..", ".."))
from naruno.lib.config_system import get_config, save_config
from naruno.lib.log import get_logger
from naruno.lib.settings_system import save_settings, temp_json, the_settings

logger = get_logger("LIB")


def naruno_import(export_location: str) -> None:
    """
    Extract a ZIP archive to the `db` folder in the main directory of the application.

    Args:
        export_location: The location of the ZIP archive to be extracted.
    """
    logger.info("Import system is started")
    backup_config = copy.copy(get_config())
    main_folder = backup_config["main_folder"]
    target_location = f"{main_folder}/db/"
    logger.debug(f"export_location: {export_location}")
    logger.debug(f"target_location: {target_location}")
    shutil.unpack_archive(export_location, target_location)

    save_config(backup_config)

    after_backup_settings = the_settings()
    for element in temp_json:
        if element not in after_backup_settings:
            after_backup_settings[element] = temp_json[element]

    save_settings(after_backup_settings)

    logger.info("Import completed")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
import os

from kivy.core.clipboard import Clipboard
from kivy.properties import StringProperty
from kivymd.uix.bottomsheet import MDListBottomSheet
from kivymd.uix.button import MDFlatButton
from kivymd.uix.gridlayout import MDGridLayout
from kivymd.uix.screen import MDScreen
from kivymd.uix.textfield import MDTextField
from kivymd_extensions.sweetalert import SweetAlert

import naruno.gui.the_naruno_gui_app
from naruno.accounts.get_balance import GetBalance
from naruno.blockchain.block.get_block import GetBlock
from naruno.gui.popup import popup
from naruno.lib.qr import qr
from naruno.lib.settings_system import change_wallet
from naruno.lib.settings_system import the_settings
from naruno.wallet.get_saved_wallet import get_saved_wallet
from naruno.wallet.wallet_create import wallet_create
from naruno.wallet.wallet_delete import wallet_delete
from naruno.wallet.wallet_import import wallet_import


class WalletScreen(MDScreen):
    pass


class WalletBox(MDGridLayout):
    cols = 2
    text = StringProperty()

    def reflesh_balance(self):

        self.text = f"Balance: {str(GetBalance(wallet_import(-1, 0)))}"

    def create_the_wallet(self):
        wallet_create(self.wallet_alert_dialog.input_results["Password"])

    def show_wallet_alert_dialog(self):
        self.wallet_alert_dialog = popup(
            title="Creating a wallet",
            target=self.create_the_wallet,
            inputs=[["Password", True]],
        )

    def callback_for_menu_items(self, *args):
        if args[0] != the_settings()["wallet"]:
            change_wallet(int(args[0]))
            self.reflesh_balance()

    def show_example_list_bottom_sheet(self):
        bottom_sheet_menu = MDListBottomSheet(radius=25, radius_from="top")
        data = {}
        all_wallets = list(get_saved_wallet())

        current_wallet = the_settings()["wallet"]
        for wallet in all_wallets:
            number = all_wallets.index(wallet)
            address = wallet_import(all_wallets.index(wallet), 3)
            if current_wallet != number:
                data[number] = address
            else:
                data[number] = f"{address} - CURRENTLY USED"

        for item in data.items():
            bottom_sheet_menu.add_item(
                f"{str(item[0])} : {item[1]}",
                lambda x, y=item[0]: self.callback_for_menu_items(y),
            )

        bottom_sheet_menu.open()

    def delete_the_wallet(self):
        saved_wallets = get_saved_wallet()
        selected_wallet_pubkey = wallet_import(int(the_settings()["wallet"]),
                                               0)
        for each_wallet in saved_wallets:
            if selected_wallet_pubkey == saved_wallets[each_wallet][
                    "publickey"]:
                change_wallet(0)
                wallet_delete(each_wallet)
                self.reflesh_balance()

    def show_delete_wallet_alert_dialog(self):
        if the_settings()["wallet"] != 0:
            self.deletewallet_alert_dialog = popup(
                title="Deleting a wallet",
                target=self.delete_the_wallet,
                type="question",
            )
        else:
            popup(title="First wallet cannot be deleted.", type="failure")

    def wallet_qr(self):
        address = wallet_import(-1, 3)
        location_of_qr = qr(address)
        popup(text=address,
              image=location_of_qr,
              height_image="400px",
              type="qr")

    def wallet_copy(self):
        Clipboard.copy(wallet_import(-1, 3))
        popup(title="The address has been copied to your clipboard.",
              type="success")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from naruno.lib.mix.merkle_root import MerkleTree


def AccountsHash(block, the_accounts):
    """
    Calculates and returns the hash of the accounts.
    """
    the_list = []
    if isinstance(the_accounts, list):
        the_list = the_accounts
        the_list = [account.get_hash()
                               for account in the_list]
    else:
        the_accounts.execute("SELECT * FROM account_list")
        the_list = the_accounts.fetchall()
        the_list = [str(account[0])+str(account[1])+str(account[2])
                               for account in the_list]

    account_list = MerkleTree(the_list).getRootHash()

    return account_list



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os
import sys
from hashlib import sha256

sys.path.append(os.path.join(os.path.dirname(__file__), "..", ".."))
from naruno.config import SIGNS_PATH
from naruno.lib.config_system import get_config
from naruno.wallet.ellipticcurve.ecdsa import Ecdsa
from naruno.wallet.ellipticcurve.privateKey import PrivateKey
from naruno.wallet.wallet_import import wallet_import


def sign(data: str, password: str) -> str:
    """
    Signs the data with the private key of the user.

    Args:
        data (str): Data to be signed
        password (str): Password of the current wallet
    """

    true_pass = wallet_import(-1, 2)
    our_pass = sha256(password.encode("utf-8")).hexdigest()
    
    if true_pass != our_pass:
        return "None"

    my_private_key = wallet_import(-1, 1, password)
    signature = Ecdsa.sign(
        data,
        PrivateKey.fromPem(my_private_key),
    ).toBase64()

    sign_json = {
        "data": data,
        "signature": signature,
        "publickey": wallet_import(-1, 0),
    }

    sign_path = os.path.join(
        SIGNS_PATH,
        sha256((signature).encode("utf-8")).hexdigest() + ".narunosign")

    os.chdir(get_config()["main_folder"])
    with open(sign_path, "w") as sign_file:
        json.dump(sign_json, sign_file)

    return sign_path


if __name__ == "__main__":
    print(sign("Ali Eren", "123"))



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import time

from naruno.blockchain.block.block_main import Block
from naruno.lib.log import get_logger

logger = get_logger("CONSENSUS_SECOND_ROUND")


def validate_main(block: Block) -> Block:
    logger.info("Validating process is started")
    block.validated = True
    block.validated_time = int(time.time())
    block.round_2 = True
    logger.debug(f"block.validated: {block.validated}")
    logger.debug(f"block.validated_time: {block.validated_time}")
    logger.debug(f"block.round_2: {block.round_2}")

    return block
